#import gettext

import os
import wx
import pickle
from  ctx import theContext as context
from ctx import logger
import app
import tran

# _globals = sys.modules[__name__]

use_splash = False


class proCxx(wx.App):
    """Clase aplicacion"""
    # theApp = None

    def OnInit(self):
        """Inicializacion de GUI"""
        global use_splash
        #self.presLan_en = gettext.translation("proCxx", "./locale",
        #     languages=['en'])
        #self.locale = wx.Locale(wxLANGUAGE_ENGLISH)
        #locale.setlocale(locale.LC_ALL, 'EN')
        #self.presLan_en.install()
        self.projects = []
        self.workspaces = []
        if use_splash:
            splash = app.AppSplashScreen()
            splash.Show()
        else:
            app.mainWindow(self)
            context.frame.Show()
            wx.GetApp().SetTopWindow(context.frame)
        stack = tran.TransactionStack(50)
        self.log = logger()
        self.log.SetFile('.proCxx.log')
        wx.Log.EnableLogging()
        stack.SetLogger(self.log)
        context.app = self
        return True

    def ExitEvenModified(self):
        """Checks about unsaved documents"""
        chain = ""
        for project in self.projects:
            if project.IsModified():
                chain += "\n" + project._name
        if len(chain) > 0:
            user = wx.MessageBox("The projects: " + chain +
            " \nhave unsaved changes.\nExit anyway?", "Alert",
                wx.YES_NO | wx.CENTER | wx.ICON_HAND, context.frame)
            if user != wx.YES:
                return False
        self.log.Dump()
        return True

    def ExistProject(self, name):
        """Comprueba si el proyecto ya esta cargado"""
        for project in self.projects:
            if project._name == name:
                return True
        return False

    def AddProject(self, project):
        """Crea un objeto de proyecto y lo incorpora a la lista interna"""
        if project not in self.projects:
            self.projects.append(project)
        return project

    def AddWorkspace(self, workspace):
        """Crea un objeto de workspace y lo incorpora a la lista interna"""
        if workspace not in self.workspaces:
            self.workspaces.append(workspace)
        return workspace

    def RemoveProject(self, project):
        """Elimina un proyecto de la lista interna"""
        while project in self.projects:
            index = self.projects.index(project)
            del self.projects[index]

    def SaveProject(self, project):
        """Almacena un proyecto"""
        if not project in self.projects:
            wx.MessageBox("Project " + project._name + " not exists", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        project_dir = project._dir.replace("//", "/")
        if not os.path.exists(project_dir):
            os.makedirs(project_dir)
            if not os.path.exists(project_dir):
                wx.MessageBox("Failed creating projec directory:\n" +
                    project_dir, "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return
        fname = (project_dir + "/" + project._name + ".pcc").replace("//", "/")
        with open(fname, "w") as f:
            pickle.dump(project, f, 2)
        project.SetModified(False)
        if project.workspace is None:
            mru = context.mru
            mru.AddFileToHistory(fname)

    def SaveSession(self):
        """Save the current session"""
        wl = self.GetSessionWorkspaces()  # workspace's path
        if not wl:
            return False
        context.config.Write('/last_session/workspaces', str(wl))
        tl = context.frame.GetViewsStatus()  # get the open views status
        context.config.Write('/last_session/views', str(tl))
        return True

    @tran.TransactionalMethod('reload last session')
    def LoadSession(self):
        """Read last session"""
        wl = context.config.Read('/last_session/workspaces', '')
        if not wl:
            return False
        wl = eval(wl)  # restore list
        if not self.SetSessionWorkspaces(wl):
            return False
        tl = context.config.Read('/last_session/views', '')
        if not tl:
            return True
        tl = eval(tl)
        if tl:
            context.frame.SetViewsStatus(tl)
        return True

    def SaveWorkspace(self, wrkspace):
        """Save the workspace and projects"""
        import model
        workspace_dir = wrkspace._dir.replace("//", "/")
        if not os.path.exists(workspace_dir):
            os.makedirs(workspace_dir)
            if not os.path.exists(workspace_dir):
                wx.MessageBox("Failed creating workspace directory:\n" +
                    workspace_dir, "Error",
                    wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
                return
        for project in wrkspace[model.Project]:
            self.SaveProject(project)
        fname = (workspace_dir + "/" + wrkspace._name + ".wrk").replace("//", "/")
        with open(fname, "w") as f:
            pickle.dump(wrkspace, f, 2)
        wrkspace.SetModified(False)
        mru = context.mru
        mru.AddFileToHistory(fname)

    def GetSessionWorkspaces(self):
        """Save full list of opened workspaces. This is a tentative.
        If some workspace is unsave, returns False. Otherwise,
        returns the workspace list."""
        files = []
        for wrk in self.workspaces:
            if wrk._modified:
                return False
            _dir = wrk._dir.replace("//", "/")
            fname = (_dir + "/" + wrk._name + ".wrk").replace("//", "/")
            files.append(fname)
        return files

    def SetSessionWorkspaces(self, files):
        """Attempts to load worskpaces"""
        status = True
        for fname in files:
            workspace = self.LoadWorkspace(fname)
            if workspace:
                tran.TransactionStack.DoLoad(workspace)
            else:
                status = False
        return status

    def LoadProject(self, fname, workspace=None, skip_compare=False):
        """Carga un proyecto existente"""
        # reload workspace must avoid waste time checking duplicates
        if not skip_compare:
            for project in self.projects:
                already_fname = (project._dir + "/" + project._name +
                    ".pcc").replace("//", "/")
                print(("compare " + already_fname + " with " + fname + "\n"))
                if already_fname == fname:
                    return None
        import traceback
        import sys
        import model
        try:
            with open(fname, "r") as f:
                project = pickle.load(f)
            if not isinstance(project, model.Project):
                return None
            if workspace:
                workspace.addChild(project)
                project._parent = workspace
            self.projects.append(project)
            project.SetModified(False)
            # check classes
            for cls in project.classes:
                if cls.errorCheckDerivatives():
                    project.SetModified(True)
            return project
        except Exception as inst:
            traceback.print_exc(file=sys.stdout)
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst
            return None

    def LoadWorkspace(self, fname):
        """Carga un workspace existente"""
        import traceback
        import sys
        import model
        for workspace in self.workspaces:
            already_fname = (workspace._dir + "/" + workspace._name +
                ".pcc").replace("//", "/")
            print(("compare " + already_fname + " with " + fname + "\n"))
            if already_fname == fname:
                return None
        try:
            with open(fname, "r") as f:
                workspace = pickle.load(f)
            if not isinstance(workspace, model.Workspace):
                return False
            self.workspaces.append(workspace)
            workspace.SetModified(False)
            mru = context.mru
            mru.AddFileToHistory(fname)
            return workspace
        except Exception as inst:
            traceback.print_exc(file=sys.stdout)
            print type(inst)     # the exception instance
            print inst.args      # arguments stored in .args
            print inst
            return None





# -*- coding: utf-8 -*-

import subprocess


def shell(command):
    """Execute a shell command and returns the value"""
    p = subprocess.Popen(command, stdout=subprocess.PIPE, shell=True)
    (output, err) = p.communicate()
    if err:
        raise OSError('shell returns {}'.format(err))
    return output


def import_dir(module, module_dir=None, project_dir=None):
    """get module info without loading"""
    if module_dir:
        try:
            s = shell('python -c "import os;os.chdir(\'{path}\');import {module} as key;print(dir(key))"'.format(
                path=module_dir, module=module))
            if s:
                return eval(s)
        except:
            pass
    if project_dir:
        try:
            s = shell('python -c "import os;os.chdir(\'{path}\');import {module} as key;print(dir(key))"'.format(
                path=project_dir, module=module))
            if s:
                return eval(s)
        except:
            pass
    try:
        s = shell('python -c "import {module} as key;print(dir(key))"'.format(module=module))
        return eval(s)
    except:
        pass
    return None



# -*- coding: utf-8 -*-
from app import cached_type
import model


def create(parent):
    """Create the classes inside this parent"""
    kwargs = {}
    kwargs['prefix'] = ''
    kwargs['parent'] = parent
    kwargs['readonly'] = True

    #containers folder
    kwargs['name'] = 'containers'
    folder = model.Folder(**kwargs)

    #array folder
    kwargs['parent'] = folder
    kwargs['name'] = 'array'
    array_folder = model.Folder(**kwargs)

    #std::array
    kwargs['parent'] = array_folder
    kwargs['name'] = 'array'
    kwargs['is_struct'] = True
    kwargs['template'] = 'class T, std::size_t N'
    kwargs['template_types'] = ['T']
    cls = model.cc.Class(**kwargs)

    kwargs['parent'] = cls
    kwargs['template'] = None
    kwargs['name'] = 'types'
    folder_types = model.Folder(**kwargs)

    kwargs['parent'] = folder_types
    kwargs['name'] = 'value_type'
    kwargs['definition'] = 'typedef T value_type;'
    value_type = model.cc.Type(**kwargs)

    kwargs['name'] = 'size_type'
    kwargs['definition'] = 'typedef std::size_t size_type;'
    size_type = model.cc.Type(**kwargs)

    kwargs['name'] = 'difference_type'
    kwargs['definition'] = 'typedef std::ptrdiff_t difference_type;'
    difference_type = model.cc.Type(**kwargs)

    kwargs['name'] = 'reference'
    kwargs['definition'] = 'typedef value_type& reference;'
    reference = model.cc.Type(**kwargs)

    kwargs['name'] = 'const_reference'
    kwargs['definition'] = 'typedef const value_type& const_reference;'
    const_reference = model.cc.Type(**kwargs)

    kwargs['name'] = 'pointer'
    kwargs['definition'] = 'typedef value_type* pointer;'
    pointer = model.cc.Type(**kwargs)

    kwargs['name'] = 'const_pointer'
    kwargs['definition'] = 'typedef const value_type* const_pointer;'
    const_pointer = model.cc.Type(**kwargs)

    kwargs['name'] = 'iterator'
    kwargs['definition'] = 'typedef RandomAmodelessIterator iterator;'
    iterator = model.cc.Type(**kwargs)

    kwargs['name'] = 'const_iterator'
    kwargs['definition'] = 'typedef Constant random access iterator const_iterator;'
    const_iterator = model.cc.Type(**kwargs)

    kwargs['name'] = 'reverse_iterator'
    kwargs['definition'] = 'typedef std::reverse_iterator<iterator> reverse_iterator;'
    reverse_iterator = model.cc.Type(**kwargs)

    kwargs['name'] = 'const_reverse_iterator'
    kwargs['definition'] = 'typedef std::reverse_iterator<const_iterator> const_reverse_iterator;'
    const_reverse_iterator = model.cc.Type(**kwargs)

    ## member functions
    kwargs['parent'] = cls
    kwargs['name'] = 'member functions'
    folder_methods = model.Folder(**kwargs)

    ## ## element access
    kwargs['parent'] = folder_methods
    kwargs['name'] = 'element access'
    folder_element_access = model.Folder(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'at'
    kwargs['type'] = model.cc.typeinst(type=reference)
    at0 = model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = at0
    kwargs['name'] = 'pos'
    kwargs['type'] = model.cc.typeinst(type=size_type)
    model.cc.Argument(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'at'
    kwargs['type'] = model.cc.typeinst(type=const_reference)
    kwargs['constmethod'] = True
    at1 = model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = at1
    kwargs['name'] = 'pos'
    kwargs['type'] = model.cc.typeinst(type=size_type)
    model.cc.Argument(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'operator []'
    kwargs['type'] = model.cc.typeinst(type=reference)
    kwargs['constmethod'] = False
    aop0 = model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = aop0
    kwargs['name'] = 'pos'
    kwargs['type'] = model.cc.typeinst(type=size_type)
    model.cc.Argument(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'operator []'
    kwargs['type'] = model.cc.typeinst(type=const_reference)
    kwargs['constmethod'] = True
    aop1 = model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = aop1
    kwargs['name'] = 'pos'
    kwargs['type'] = model.cc.typeinst(type=size_type)
    model.cc.Argument(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'front'
    kwargs['type'] = model.cc.typeinst(type=reference)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'front'
    kwargs['type'] = model.cc.typeinst(type=const_reference)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'back'
    kwargs['type'] = model.cc.typeinst(type=reference)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'back'
    kwargs['type'] = model.cc.typeinst(type=const_reference)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'data'
    kwargs['type'] = model.cc.typeinst(type=
    cached_type(parent.project, '@'), type_alias='T', const=True, ptr=True)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_element_access
    kwargs['name'] = 'data'
    kwargs['type'] = model.cc.typeinst(type=
    cached_type(parent.project, '@'), type_alias='T', const=True, ptr=True)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    ## ## iterators
    kwargs['parent'] = folder_methods
    kwargs['name'] = 'iterators'
    folder_iterators = model.Folder(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'begin'
    kwargs['type'] = model.cc.typeinst(type=iterator)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'begin'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'cbegin'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'end'
    kwargs['type'] = model.cc.typeinst(type=iterator)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'end'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'cend'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'rbegin'
    kwargs['type'] = model.cc.typeinst(type=iterator)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'rbegin'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'crbegin'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'rend'
    kwargs['type'] = model.cc.typeinst(type=iterator)
    kwargs['constmethod'] = False
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'rend'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = folder_iterators
    kwargs['name'] = 'crend'
    kwargs['type'] = model.cc.typeinst(type=const_iterator)
    kwargs['constmethod'] = True
    model.cc.MemberMethod(**kwargs)

    ## ## capacity
    kwargs['parent'] = folder_methods
    kwargs['name'] = 'capacity'
    folder_capacity = model.Folder(**kwargs)

    kwargs['parent'] = folder_capacity
    kwargs['name'] = 'fill'
    kwargs['type'] = model.cc.typeinst(type=
        cached_type(parent.project, 'void'))
    kwargs['constmethod'] = False
    fill = model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = fill
    kwargs['name'] = 'value'
    kwargs['type'] = model.cc.typeinst(type=
        cached_type(parent.project, '@'), type_alias='T', const=True, ref=True)
    model.cc.Argument(**kwargs)

    kwargs['parent'] = folder_capacity
    kwargs['name'] = 'swap'
    kwargs['type'] = model.cc.typeinst(type=
        cached_type(parent.project, 'void'))
    swap = model.cc.MemberMethod(**kwargs)

    kwargs['parent'] = swap
    kwargs['name'] = 'other'
    kwargs['type'] = model.cc.typeinst(type=cls, ref=True)
    model.cc.Argument(**kwargs)

    # Create module for related globals
    kwargs['parent'] = array_folder
    kwargs['name'] = 'non-member-functions '
    non_members = model.cc.Module(**kwargs)

    for op in ['==', '!=', '<', '<=', '>', '>=']:
        kwargs['parent'] = non_members
        kwargs['name'] = 'operator {op}'.format(op=op)
        kwargs['template'] = 'class T, std::size_t N'
        kwargs['template_types'] = ['T']
        kwargs['type'] = model.cc.typeinst(type=cached_type(parent.project, 'bool'))
        operator = model.cc.Function(**kwargs)

        kwargs['parent'] = operator
        kwargs['name'] = 'lhs'
        kwargs['type'] = model.cc.typeinst(type=cls, const=True, ref=True)
        model.cc.Argument(**kwargs)

        kwargs['parent'] = operator
        kwargs['name'] = 'rhs'
        kwargs['type'] = model.cc.typeinst(type=cls, const=True, ref=True)
        model.cc.Argument(**kwargs)

    # Helpers
    kwargs['parent'] = array_folder
    kwargs['name'] = 'helper classes'
    helpers = model.Folder(**kwargs)

    kwargs['parent'] = helpers
    kwargs['name'] = 'tuple_size'
    kwargs['is_struct'] = False
    kwargs['template'] = 'class T, std::size_t N'
    kwargs['template_types'] = ['T']
    model.cc.Class(**kwargs)

    kwargs['parent'] = helpers
    kwargs['name'] = 'tuple_element'
    kwargs['template'] = 'std::size_t I, class T, std::size_t N'
    kwargs['template_types'] = ['T']
    model.cc.Class(**kwargs)

    #vector folder
    kwargs['parent'] = folder
    kwargs['name'] = 'vector'
    vector_folder = model.Folder(**kwargs)

    #std::vector
    kwargs['parent'] = vector_folder
    kwargs['name'] = 'vector'
    kwargs['template'] = 'class T, class Allocator = std::allocator<T>'
    kwargs['template_types'] = ['T']
    vector = model.cc.Class(**kwargs)





# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""
import model
import app.resources as rc
from tran import TransactionStack


class Friendship(model.TComponent):
    """Implements friendship representation"""
    def __init__(self, **kwargs):
        """ Initialice the friendship. Required parameters:
            target: the friend class
            parent: the child class
        """
        assert 'target' in kwargs
        assert 'parent' in kwargs
        self._target = kwargs['target']
        if 'name' not in kwargs:
            kwargs['name'] = self._target._name
        super(Friendship, self).__init__(**kwargs)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Friendship, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['target'] = self._target
        kwargs.update(super(Friendship, self).get_kwargs())
        return kwargs

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("friend")

    @property
    def label(self):
        """Get tree label"""
        return "friend " + self._target.reference

    def OnUndoRedoChanged(self):
        """Make changes in the model as result of change"""
        super(Friendship, self).OnUndoRedoChanged()
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)




"""Subclass of NewWorkspace, which is generated by wxFormBuilder."""

import os
import wx
import wxx
import app
from app import resources as rc


# Implementing NewWorkspace
class WorkspaceDialog(app.NewWorkspace):
    """
    This dialog allows to add workspaces
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container=None):
        """initialization"""
        super(WorkspaceDialog, self).__init__(parent)
        # container es la clase destino
        self.container = container
        self._original = ""
        self._autodir = True
        icon = wx.EmptyIcon()
        icon.CopyFromBitmap(rc.GetBitmap("workspace"))
        self.SetIcon(icon)

    # Handlers for NewWorkspace events.
    def OnChangeWorkspaceName(self, event):
        """Handle workspace name change"""
        if self._autodir:
            name = self.m_textCtrl1.GetValue()
            std = wx.StandardPaths.Get().GetDocumentsDir()
            #create a custom subdirectory
            self.m_textCtrl9.SetValue(os.path.join(std, name))

    def OnChooseDir(self, event):
        """Launch subdialog for choosing dir"""
        dialog = wx.DirDialog(self, "Select directory",
            self.m_textCtrl9.GetValue(), wx.DD_DIR_MUST_EXIST)
        if dialog.ShowModal() == wx.ID_OK:
            self.m_textCtrl9.SetValue(dialog.GetPath())
            self._autodir = False

    def Validate(self):
        """validates dialog when ok request"""
        self._name = self.m_textCtrl1.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Workspace name must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._base = self.m_textCtrl9.GetValue()
        if len(self._name) == 0:
            wx.MessageBox("Invalid workspace base directory", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self._note = self.m_richText3.GetValue()
        return True

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for object creation"""
        kwargs = {}
        kwargs['name'] = self._name
        kwargs['dir'] = self._base
        kwargs['note'] = self._note
        return kwargs

    def CopyAttributes(self, workspace):
        """Copy attributes to method"""
        workspace._name = self._name
        workspace._dir = self._base
        workspace.note = self._note

    def SetAttributes(self, workspace):
        """Setup attributes for editing already method"""
        self.SetTitle("Edit workspace")
        self.m_textCtrl1.SetValue(workspace._name)
        self.m_textCtrl9.SetValue(workspace._dir)
        self.m_richText3.SetValue(workspace.note)

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Handle OnOk"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
import wx
import model
import app.resources as rc
from ctx import theContext as context
import model.decorator as ctx
import tran


class MemberMethod(model.Member):
    """Implements member method"""
    context_container = True
    argument_container = True
    import_container = True

    # visual methods
    @tran.TransactionalMethod('move method {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_member_container
        if not target or self.inner_class != target.inner_class or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        self._staticmethod = kwargs.get('static_method', False)
        self._classmethod = kwargs.get('class_method', False)
        self._property = kwargs.get('property', False)
        self._content = kwargs.get('content', "")
        super(MemberMethod, self).__init__(**kwargs)
        if not self._staticmethod and not kwargs.get('raw', False):
            from Argument import Argument
            if self._property or not self._classmethod:
                if not self.ExistArgumentNamed('self'):
                    Argument(parent=self, name='self')
            else:
                if not self.ExistArgumentNamed('self'):
                    Argument(parent=self, name='cls')
        if self._staticmethod:
            self._staticmethod = model.py.Decorator(parent=self, name='staticmethod')
        if self._classmethod:
            self._classmethod = model.py.Decorator(parent=self, name='classmethod')
        if self._property:
            self._property = model.py.Decorator(parent=self, name='property')
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()

    def Delete(self):
        """Handle delete"""
        k = self.inner_module or self.inner_package
        super(MemberMethod, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['static_method'] = bool(self._staticmethod)
        kwargs['class_method'] = bool(self._classmethod)
        kwargs['property'] = bool(self._property)
        kwargs['content'] = self._content
        kwargs.update(super(MemberMethod, self).get_kwargs())
        return kwargs

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the member method declaration"""
        pass

    @ctx.ContextImplementation()
    def WriteCode(self, f):
        """Write code to file"""
        pass

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(MemberMethod, self).OnUndoRedoChanged()
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            index = book.GetPageIndex(self._pane)
            book.SetPageText(index, self.tab_label)
            book.SetPageBitmap(index, self.GetTabBitmap())
            if self._pane.m_editor.GetText() != self._content:
                self._pane.m_editor.SetText(self._content)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            p = self._pane
            delattr(self, '_pane')
            p.Commit()
            p.PreDelete()    # avoid gtk-critical
            self._paneIndex = book.GetPageIndex(p)
            book.DeletePage(self._paneIndex)
        super(MemberMethod, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        import activity.models.ui.pane as pane
        super(MemberMethod, self).OnUndoRedoAdd()
        if hasattr(self, '_paneIndex'):
            book = context.frame.docBook
            p = pane.PyMethodPane(book, context.frame, self)
            self._pane = p
            book.InsertPage(self._paneIndex, p, self.tab_label,
                False, self.bitmap_index)
            delattr(self, '_paneIndex')

    def ExportPythonCode(self, wf):
        """Write code"""
        # first, write decorators, if any
        for deco in self[model.py.Decorator]:
            deco.ExportPythonCode(wf)
        wf.openbrace(self.declare)
        wf.writecomment(self._note)
        wf.writeblock(self._content)
        wf.closebrace()
        wf.writenl()

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap("py_method")

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("py_method")

    @property
    def tree_label(self):
        """Get tree label"""
        from Argument import Argument
        from ArgsArgument import ArgsArgument
        from KwArgsArgument import KwArgsArgument
        alist = ', '.join(arg.label for arg in
            self[Argument] + self[ArgsArgument] + self[KwArgsArgument])
        return '{self._name}({alist})'.format(self=self, alist=alist)

    @property
    def tab_label(self):
        """Get tab label"""
        from Argument import Argument
        from ArgsArgument import ArgsArgument
        from KwArgsArgument import KwArgsArgument
        alist = ', '.join(arg.label for arg in
            self[Argument] + self[ArgsArgument] + self[KwArgsArgument])
        return '{self._name}({alist})'.format(self=self, alist=alist)

    @property
    def declare(self):
        """Get tab label"""
        from Argument import Argument
        from ArgsArgument import ArgsArgument
        from KwArgsArgument import KwArgsArgument
        alist = ', '.join(arg.label for arg in
            self[Argument] + self[ArgsArgument] + self[KwArgsArgument])
        return 'def {self._name}({alist}):'.format(self=self, alist=alist)

    def ExistArgumentNamed(self, name):
        """Check about the existence of an argument"""
        from Argument import Argument
        from ArgsArgument import ArgsArgument
        from KwArgsArgument import KwArgsArgument
        return name in [x._name for x in self(Argument, ArgsArgument, KwArgsArgument)]




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""


import copy
import re
import wx
import model
import model.decorator as ctx
import tran


from ctx import theContext as context
import app.resources as rc


class Constructor(model.Member):
    """Implements ctor method"""
    context_container = True
    argument_container = True

    # visual methods
    @tran.TransactionalMethod('move constructor {}')
    def drop(self, to):
        """Drops class inside project or another folder """
        target = to.inner_member_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        # the name must be allways the same of the class
        kwargs['name'] = kwargs['parent'].inner_class.name
        self._access = kwargs.get('access', "public")
        self._explicit = kwargs.get('explicit', False)
        self._inline = kwargs.get('inline', False)
        self._calling = kwargs.get('calling', False)
        self._callingText = kwargs.get('callspec', False)
        self._init = kwargs.get('init', "")
        self._content = kwargs.get('content', "")
        self._preferred = kwargs.get('preferred', False)
        self._initTouched = False
        self._argsTouched = False
        self._bodyTouched = False
        self._template = kwargs.get('template', None)
        self._template_types = kwargs.get('template_types', [])
        super(Constructor, self).__init__(**kwargs)
        if kwargs.get('autoargs', True):
            self.AutoArgs()
            self.AutoInit(False)
            self.UpdateInitCall(False)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Constructor, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['access'] = self._access
        kwargs['explicit'] = self._explicit
        kwargs['inline'] = self._inline
        kwargs['calling'] = self._calling
        kwargs['callspec'] = self._callingText
        kwargs['init'] = self._init
        kwargs['content'] = self._content
        kwargs['preferred'] = self._preferred
        kwargs.update(super(Constructor, self).get_kwargs())
        return kwargs

    @ctx.ContextDeclaration()
    def WriteDeclaration(self, pf):
        """Write the ctor declaration"""
        self.inner_class._ensure_access(pf, self._access)
        pf.writeln(self.declare)
        if self._inline:
            self.outer_class._inlines.append(self)

    def AutoArgs(self):
        """Computes required arguments"""
        if self._argsTouched:
            return
        s = copy.copy(self[model.cc.Argument])
        for arg in s:
            arg.Delete()
        args = self.inner_class.GetClassArguments()
        used = []
        for x in args:
            if x._name in used:
                continue
            used.append(x._name)
            if type(x) is model.cc.RelationFrom:
                ti = model.cc.typeinst(type=x.GetTo().parent, ref=True, const=True)
            else:
                ti = copy.copy(x._typei)
            model.cc.Argument(parent=self, name=x._name, type=ti)

    def UpdateInitCall(self, save=True):
        """Updates the call to init method"""
        if save:
            self.SaveState()
        method = self.inner_class.init_methods
        mask = r'\t__init__\s*\(.*\);\n'
        if not method:
            #if there are not init methods, remove the call
            self._content = re.sub(mask, '', self._content)
            return
        method = method[0]
        #create the call
        callstr = '\t__init__('
        for arg in method[model.cc.Argument]:
            callstr += arg._name + ','
        callstr = callstr[:-1] + ');\n'
        #we need to find the method call
        if re.search(mask, self._content):
            self._content = re.sub(mask, callstr, self._content)
        else:
            self._content = callstr + self._content

    def AutoInit(self, save=True):
        """Computes the init segment"""
        if self._initTouched:
            return
        if save:
            self.SaveState()
        # get inheritence classes
        cls = self.inner_class
        self_cls = lambda x: x.inner_class == cls
        inh = [x._ancestor for x in cls(model.cc.Inheritance, filter=self_cls, cut=True)]
        # get the virtual base classes
        vinh = cls.virtual_bases
        # remove virtuals from normal inheritance
        inh = [x for x in inh if x not in vinh]
        #create a line array
        l = []
        if len(vinh) > 0:
            # l.append("\t//virtual inheritances")
            for x in vinh:
                ctor = x.GetPreferredConstructor()
                s = ''
                if ctor is not None:
                    for arg in ctor[model.cc.Argument]:
                        if len(s) > 0:
                            s = s + ",{}".format(arg._name)
                        else:
                            s = s + arg._name
                l.append("\t{}({})".format(x._name, s))
        if len(inh) > 0:
            # l.append("\t//direct bases")
            for x in inh:
                ctor = x.GetPreferredConstructor()
                s = ''
                if ctor is not None:
                    for arg in ctor[model.cc.Argument]:
                        if len(s) > 0:
                            s = s + ",{}".format(arg._name)
                        else:
                            s = s + arg._name
                l.append("\t{}({})".format(x._name, s))
        #ok, now initialize members
        members = cls(model.cc.MemberData, filter=lambda x: self_cls(x) and not x._static)
        if len(members) > 0:
            # l.append("\t//data members")
            for x in members:
                l.append("\t{}({})".format(cls._memberPrefix + x._name,
                    x.GetInitializer()))
        #Ok, now, generate initialization
        self._init = ''
        sep = ':'
        for s in l:
            self._init += "{}{}\n".format(sep, s)
            sep = ','
        #Update visuals
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            index = book.GetPageIndex(self._pane)
            book.SetPageText(index, self.tab_label)
            book.SetPageBitmap(index, self.GetTabBitmap())
            self._pane.m_init.SetText(self._init)

    def SetSerial(self, flag):
        """Set the serial flag indicating that this is a serializer ctor"""
        if flag:
            cl = self.inner_class.GetSerialConstructor()
            assert cl is None or cl is self
        self._serial = flag

    def SetPreferred(self, flag):
        """Set the preferred flag"""
        if flag:
            # if any one ctor has preferred flag, simply
            # save his state and change
            cls = self.inner_class
            criteria = lambda x: x.inner_class == cls and x._preferred
            pre = cls(model.cc.Constructor, filter=criteria)
            assert len(pre) < 2
            if len(pre):
                pre[0].SaveState()
                pre[0]._preferred = False
        self._preferred = flag

    def IsPreferred(self):
        """Get the preferred flag"""
        return self._preferred

    @ctx.ContextImplementation()
    def WriteCode(self, f):
        """Write code to file"""
        self.WriteComment(f)
        f.writeln(self.implement)
        if len(self._init) > 0:
            f.writeln(self._init)
        f.openbrace()
        f.writeln(self._content)
        f.closebrace()

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Constructor, self).OnUndoRedoChanged()
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            index = book.GetPageIndex(self._pane)
            book.SetPageText(index, self.name)
            book.SetPageBitmap(index, self.GetTabBitmap())
            if self._pane.m_init.GetText() != self._init:
                self._pane.m_init.SetText(self._init)
            if self._pane.m_editor.GetText() != self._content:
                self._pane.m_editor.SetText(self._content)
        if not tran.TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            pane = self._pane
            delattr(self, '_pane')
            # we are not able to commit here!
            # pane.Commit()
            pane.PreDelete()    # avoid gtk-critical
            self._paneIndex = book.GetPageIndex(pane)
            book.DeletePage(self._paneIndex)
        super(Constructor, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(Constructor, self).OnUndoRedoAdd()
        if hasattr(self, '_paneIndex'):
            book = context.frame.docBook
            from pane.ConstructorPane import ConstructorPane
            pane = ConstructorPane(book, context.frame, self)
            self._pane = pane
            book.InsertPage(self._paneIndex, pane, self.tab_label,
                False, self.bitmap_index)
            delattr(self, '_paneIndex')

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        from app.resources import GetBitmap
        return GetBitmap("constructor", self._access)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('constructor', self._access)

    @property
    def tree_label(self):
        """Get tree label"""
        from Argument import Argument
        argl = ', '.join(arg.label for arg in self[Argument])
        prolog = (self._inline and "inline ") or ""
        return '{prolog}{name}({argl})'.format(prolog=prolog, name=self._name, argl=argl)

    @property
    def tab_label(self):
        """Get tree label"""
        from Argument import Argument
        argl = ', '.join(arg.label for arg in self[Argument])
        return '{name}({argl})'.format(name=self._name, argl=argl)

    @property
    def declare(self):
        """Get the string for declaration."""
        from Argument import Argument
        argl = ', '.join(arg.declare for arg in self[Argument])
        return '{self._name}({args});'.format(
            self=self, args=argl)

    @property
    def implement(self, no_default=False):
        """Get the string for implementation"""
        from Argument import Argument
        argl = ', '.join(arg.implement for arg in self[Argument])
        return '{self.scope}{self._name}({args})'.format(
            self=self, args=argl)

    def ExistArgumentNamed(self, name):
        """Returns information about argument existence"""
        from Argument import Argument
        return self.hasChild(name=name, type=Argument)

    @property
    def template_types(self):
        """Returns the list of nested template types"""
        # we need to add types from template nested classes
        lt = self._template_types
        nt = super(Constructor, self).template_types
        lt.extend([x for x in nt if x not in self._template_types])
        return lt

        self._argsTouched = False
        self._bodyTouched = False

    def __setstate__(self, d):
        """Load pickle context.
        This will add some missing attributes in previous releases."""
        d['_argsTouched'] = d.get('_argsTouched', False)
        d['_bodyTouched'] = d.get('_bodyTouched', False)
        self.__dict__ = d



# -*- coding: utf-8 -*- 

###########################################################################
## Python code generated with wxFormBuilder (version Jun  6 2014)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc
import wx.richtext
import wxx
import wx.propgrid as pg

###########################################################################
## Class NewTask
###########################################################################

class NewTask ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"New Task", pos = wx.DefaultPosition, size = wx.Size( 518,544 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer6 = wx.FlexGridSizer( 5, 1, 0, 0 )
		fgSizer6.AddGrowableCol( 0 )
		fgSizer6.AddGrowableRow( 3 )
		fgSizer6.SetFlexibleDirection( wx.BOTH )
		fgSizer6.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		fgSizer7 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer7.AddGrowableCol( 1 )
		fgSizer7.AddGrowableRow( 0 )
		fgSizer7.SetFlexibleDirection( wx.BOTH )
		fgSizer7.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText4 = wx.StaticText( self, wx.ID_ANY, u"Short description:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText4.Wrap( -1 )
		fgSizer7.Add( self.m_staticText4, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		self.m_textCtrl2 = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer7.Add( self.m_textCtrl2, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )
		
		
		fgSizer6.Add( fgSizer7, 0, wx.EXPAND|wx.ALL, 5 )
		
		fgSizer5 = wx.FlexGridSizer( 3, 4, 0, 0 )
		fgSizer5.AddGrowableCol( 1 )
		fgSizer5.AddGrowableCol( 3 )
		fgSizer5.SetFlexibleDirection( wx.BOTH )
		fgSizer5.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText3 = wx.StaticText( self, wx.ID_ANY, u"type:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText3.Wrap( -1 )
		fgSizer5.Add( self.m_staticText3, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		m_choice2Choices = [ u"development", u"modification", u"improvement", u"fix" ]
		self.m_choice2 = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, m_choice2Choices, 0 )
		self.m_choice2.SetSelection( 0 )
		fgSizer5.Add( self.m_choice2, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticText41 = wx.StaticText( self, wx.ID_ANY, u"created:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText41.Wrap( -1 )
		fgSizer5.Add( self.m_staticText41, 0, wx.ALL|wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_staticText6 = wx.StaticText( self, wx.ID_ANY, u"MyLabel", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText6.Wrap( -1 )
		fgSizer5.Add( self.m_staticText6, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_staticText48 = wx.StaticText( self, wx.ID_ANY, u"status:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText48.Wrap( -1 )
		fgSizer5.Add( self.m_staticText48, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		m_choice1Choices = [ u"pending", u"working", u"done" ]
		self.m_choice1 = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, m_choice1Choices, 0 )
		self.m_choice1.SetSelection( 0 )
		fgSizer5.Add( self.m_choice1, 1, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticText5 = wx.StaticText( self, wx.ID_ANY, u"started:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText5.Wrap( -1 )
		fgSizer5.Add( self.m_staticText5, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		self.m_staticText7 = wx.StaticText( self, wx.ID_ANY, u"MyLabel", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText7.Wrap( -1 )
		fgSizer5.Add( self.m_staticText7, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_staticText12 = wx.StaticText( self, wx.ID_ANY, u"priority:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText12.Wrap( -1 )
		fgSizer5.Add( self.m_staticText12, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		m_choice3Choices = [ u"Low", u"Normal", u"High", u"Critical" ]
		self.m_choice3 = wx.Choice( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, m_choice3Choices, 0 )
		self.m_choice3.SetSelection( 1 )
		fgSizer5.Add( self.m_choice3, 0, wx.ALL|wx.EXPAND, 5 )
		
		self.m_staticText71 = wx.StaticText( self, wx.ID_ANY, u"finished:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText71.Wrap( -1 )
		fgSizer5.Add( self.m_staticText71, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_staticText72 = wx.StaticText( self, wx.ID_ANY, u"MyLabel", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText72.Wrap( -1 )
		fgSizer5.Add( self.m_staticText72, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		
		fgSizer6.Add( fgSizer5, 1, wx.EXPAND, 5 )
		
		fgSizer61 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer61.AddGrowableCol( 1 )
		fgSizer61.SetFlexibleDirection( wx.BOTH )
		fgSizer61.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText10 = wx.StaticText( self, wx.ID_ANY, u"reference:", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText10.Wrap( -1 )
		fgSizer61.Add( self.m_staticText10, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		self.m_staticText11 = wx.StaticText( self, wx.ID_ANY, u"none", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText11.Wrap( -1 )
		fgSizer61.Add( self.m_staticText11, 0, wx.ALL|wx.EXPAND|wx.ALIGN_CENTER_VERTICAL, 5 )
		
		
		fgSizer6.Add( fgSizer61, 1, wx.EXPAND, 5 )
		
		sbSizer9 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Documentation" ), wx.VERTICAL )
		
		self.m_richText3 = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0|wx.HSCROLL|wx.SUNKEN_BORDER|wx.VSCROLL|wx.WANTS_CHARS )
		sbSizer9.Add( self.m_richText3, 1, wx.EXPAND|wx.ALL, 5 )
		
		
		fgSizer6.Add( sbSizer9, 1, wx.EXPAND, 5 )
		
		fgSizer22 = wx.FlexGridSizer( 1, 3, 0, 0 )
		fgSizer22.AddGrowableCol( 0 )
		fgSizer22.SetFlexibleDirection( wx.BOTH )
		fgSizer22.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer22.Add( self.m_info, 0, wx.ALL, 5 )
		
		self.m_button5 = wx.Button( self, wx.ID_CANCEL, u"Cancel", wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer22.Add( self.m_button5, 0, wx.ALL, 5 )
		
		self.m_button6 = wx.Button( self, wx.ID_OK, u"Ok", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_button6.SetDefault() 
		fgSizer22.Add( self.m_button6, 0, wx.ALL, 5 )
		
		
		fgSizer6.Add( fgSizer22, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer6 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_button6.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnOK( self, event ):
		event.Skip()
	

###########################################################################
## Class TasksView
###########################################################################

class TasksView ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 510,801 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer41 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer41.AddGrowableCol( 0 )
		fgSizer41.AddGrowableRow( 0 )
		fgSizer41.SetFlexibleDirection( wx.BOTH )
		fgSizer41.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_splitter = wx.SplitterWindow( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.SP_3D )
		self.m_splitter.SetSashGravity( 1 )
		self.m_splitter.Bind( wx.EVT_IDLE, self.m_splitterOnIdle )
		self.m_splitter.SetMinimumPaneSize( 50 )
		
		self.m_treePane = wx.Panel( self.m_splitter, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TAB_TRAVERSAL|wx.WANTS_CHARS )
		fgSizer8 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer8.AddGrowableCol( 0 )
		fgSizer8.AddGrowableRow( 0 )
		fgSizer8.SetFlexibleDirection( wx.BOTH )
		fgSizer8.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_tree = wxx.TreeCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TR_HAS_BUTTONS|wx.TR_HIDE_ROOT|wx.TR_SINGLE | wx.NO_BORDER)
		fgSizer8.Add( self.m_tree, 0, wx.EXPAND, 5 )
		
		
		self.m_treePane.SetSizer( fgSizer8 )
		self.m_treePane.Layout()
		fgSizer8.Fit( self.m_treePane )
		self.m_splitter.Initialize( self.m_treePane )
		fgSizer41.Add( self.m_splitter, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer41 )
		self.Layout()
		
		# Connect Events
		self.m_tree.Bind( wx.EVT_RIGHT_DOWN, self.OnTasksMenu )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnTasksMenu( self, event ):
		event.Skip()
	
	def m_splitterOnIdle( self, event ):
		self.m_splitter.SetSashPosition( 553 )
		self.m_splitter.Unbind( wx.EVT_IDLE )
	

###########################################################################
## Class Info
###########################################################################

class Info ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,150 ), style = wx.TAB_TRAVERSAL )
		
		fgSizer247 = wx.FlexGridSizer( 2, 1, 0, 0 )
		fgSizer247.AddGrowableCol( 0 )
		fgSizer247.AddGrowableRow( 0 )
		fgSizer247.SetFlexibleDirection( wx.BOTH )
		fgSizer247.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_propertyGrid = pg.PropertyGrid(self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.propgrid.PG_DEFAULT_STYLE|wx.SIMPLE_BORDER)
		fgSizer247.Add( self.m_propertyGrid, 0, wx.EXPAND|wx.BOTTOM|wx.RIGHT|wx.LEFT, 5 )
		
		self.m_text = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0|wx.HSCROLL|wx.SIMPLE_BORDER|wx.VSCROLL|wx.WANTS_CHARS )
		self.m_text.SetMinSize( wx.Size( -1,100 ) )
		
		fgSizer247.Add( self.m_text, 1, wx.EXPAND|wx.BOTTOM|wx.RIGHT|wx.LEFT, 5 )
		
		
		self.SetSizer( fgSizer247 )
		self.Layout()
	
	def __del__( self ):
		pass
	

###########################################################################
## Class NewFolder
###########################################################################

class NewFolder ( wx.Dialog ):
	
	def __init__( self, parent ):
		wx.Dialog.__init__ ( self, parent, id = wx.ID_ANY, title = u"New tasks folder", pos = wx.DefaultPosition, size = wx.Size( 423,390 ), style = wx.DEFAULT_DIALOG_STYLE )
		
		self.SetSizeHintsSz( wx.DefaultSize, wx.DefaultSize )
		
		fgSizer6 = wx.FlexGridSizer( 3, 1, 0, 0 )
		fgSizer6.AddGrowableCol( 0 )
		fgSizer6.AddGrowableRow( 1 )
		fgSizer6.SetFlexibleDirection( wx.BOTH )
		fgSizer6.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		fgSizer7 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer7.AddGrowableCol( 1 )
		fgSizer7.SetFlexibleDirection( wx.BOTH )
		fgSizer7.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_staticText4 = wx.StaticText( self, wx.ID_ANY, u"Name", wx.DefaultPosition, wx.DefaultSize, 0 )
		self.m_staticText4.Wrap( -1 )
		fgSizer7.Add( self.m_staticText4, 0, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.ALIGN_RIGHT, 5 )
		
		self.m_textCtrl2 = wx.TextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0 )
		fgSizer7.Add( self.m_textCtrl2, 1, wx.ALL|wx.ALIGN_CENTER_VERTICAL|wx.EXPAND, 5 )
		
		
		fgSizer6.Add( fgSizer7, 0, wx.EXPAND|wx.ALL, 5 )
		
		sbSizer9 = wx.StaticBoxSizer( wx.StaticBox( self, wx.ID_ANY, u"Documentation" ), wx.VERTICAL )
		
		self.m_richText3 = wx.richtext.RichTextCtrl( self, wx.ID_ANY, wx.EmptyString, wx.DefaultPosition, wx.DefaultSize, 0|wx.HSCROLL|wx.SUNKEN_BORDER|wx.VSCROLL|wx.WANTS_CHARS )
		sbSizer9.Add( self.m_richText3, 1, wx.EXPAND|wx.ALL, 5 )
		
		
		fgSizer6.Add( sbSizer9, 1, wx.EXPAND, 5 )
		
		fgSizer162 = wx.FlexGridSizer( 1, 2, 0, 0 )
		fgSizer162.AddGrowableCol( 1 )
		fgSizer162.SetFlexibleDirection( wx.BOTH )
		fgSizer162.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_info = wx.BitmapButton( self, wx.ID_ANY, wx.ArtProvider.GetBitmap( wx.ART_TIP, wx.ART_BUTTON ), wx.DefaultPosition, wx.DefaultSize, wx.BU_AUTODRAW|wx.NO_BORDER )
		fgSizer162.Add( self.m_info, 0, wx.ALL, 5 )
		
		m_sdbSizer2 = wx.StdDialogButtonSizer()
		self.m_sdbSizer2OK = wx.Button( self, wx.ID_OK )
		m_sdbSizer2.AddButton( self.m_sdbSizer2OK )
		self.m_sdbSizer2Cancel = wx.Button( self, wx.ID_CANCEL )
		m_sdbSizer2.AddButton( self.m_sdbSizer2Cancel )
		m_sdbSizer2.Realize();
		
		fgSizer162.Add( m_sdbSizer2, 1, wx.EXPAND|wx.TOP|wx.BOTTOM, 5 )
		
		
		fgSizer6.Add( fgSizer162, 1, wx.EXPAND, 5 )
		
		
		self.SetSizer( fgSizer6 )
		self.Layout()
		
		self.Centre( wx.BOTH )
		
		# Connect Events
		self.m_sdbSizer2OK.Bind( wx.EVT_BUTTON, self.OnOK )
	
	def __del__( self ):
		pass
	
	
	# Virtual event handlers, overide them in your derived class
	def OnOK( self, event ):
		event.Skip()
	




# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:08:46 2013

@author: mel
"""
import model
import app.resources as rc
from ctx import theContext as context
import tran
from tran import TransactionStack


class Function(model.TComponent):
    """Implements member function"""
    context_container = True
    argument_container = True

    # visual methods
    @tran.TransactionalMethod('move function {}')
    def drop(self, to):
        """Drops datamember inside project or another folder """
        target = to.inner_function_container
        if not target or self.project != target.project:
            return False  # avoid move classes between projects
        index = 0
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        self._typei = kwargs['type']
        self._static = kwargs.get('static', False)
        self._inline = kwargs.get('inline', False)
        self._content = kwargs.get('content', "")
        self._template = kwargs.get('template', None)
        self._template_types = kwargs.get('template_types', [])
        super(Function, self).__init__(**kwargs)
        k = self.outer_class or self.outer_module
        if k:
            k.ExportCppCodeFiles(force=True)

    def Delete(self):
        """Handle delete"""
        k = self.outer_class or self.outer_module
        super(Function, self).Delete()
        if k:
            k.ExportCppCodeFiles(force=True)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['type'] = self._typei
        kwargs['static'] = self._static
        kwargs['inline'] = self._inline
        kwargs['content'] = self._content
        kwargs['template'] = self._template
        kwargs['template_types'] = self._template_types
        kwargs.update(super(Function, self).get_kwargs())
        return kwargs

    def WriteCode(self, f):
        """Write code to file"""
        if len(self._note):
            f.opencomment()
            f.writeln(self._note)
            f.closecomment()
        f.writeln(self.implement)
        f.openbrace()
        f.writeln(self._content)
        f.closebrace()

    def OnUndoRedoChanged(self):
        """Update from app"""
        super(Function, self).OnUndoRedoChanged()
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            index = book.GetPageIndex(self._pane)
            book.SetPageText(index, self.tab_labe)
            book.SetPageBitmap(index, self.GetTabBitmap())
            if self._pane.m_editor.GetText() != self._content:
                self._pane.m_editor.SetText(self._content)
        if not TransactionStack.InUndoRedo():
            k = self.outer_class or self.outer_module
            if k:
                k.ExportCppCodeFiles(force=True)

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        if hasattr(self, '_pane') and not self._pane is None:
            book = context.frame.docBook
            p = self._pane
            delattr(self, '_pane')
            # we are not able to commit here!
            # pane.Commit()
            p.PreDelete()    # avoid gtk-critical
            self._paneIndex = book.GetPageIndex(p)
            book.DeletePage(self._paneIndex)
        super(Function, self).OnUndoRedoRemoving(root)

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        import pane
        super(Function, self).OnUndoRedoAdd()
        if hasattr(self, '_paneIndex'):
            book = context.frame.docBook
            p = pane.MethodPane(book, context.frame, self)
            self._pane = p
            book.InsertPage(self._paneIndex, p, self.tab_label,
                False, self.bitmap_index)
            delattr(self, '_paneIndex')

    def GetTabBitmap(self):
        """Get the bitmap for tab control"""
        return rc.GetBitmap("function")

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("function")

    @property
    def tab_label(self):
        """Get tree label"""
        from Argument import Argument
        argl = ', '.join(arg.label for arg in self[Argument])
        stype = (self._template and 'template<{}> '.format(self._template)) or ''
        return format("{stype}{name}({argl})".format(stype=stype, name=self._name, argl=argl))

    @property
    def tree_label(self):
        """Get tree label"""
        if self._template:
            stype = 'template<{}> '.format(self._template)
        else:
            stype = ''
            if self._inline:
                stype = "inline " + stype
        stype += str(self._typei)
        from Argument import Argument
        argl = ', '.join(arg.label for arg in self[Argument])
        return stype.format(self._name + "(" + argl + ")")

    @property
    def declare(self):
        """Gets tab label"""
        from Argument import Argument
        s = str(self._typei)
        argl = ', '.join(arg.declare for arg in self[Argument])
        return s.format(" {self._name}({argl})".format(self=self, argl=argl)) + ";"

    def implement(self, no_default=False):
        """Gets tab label"""
        from Argument import Argument
        argl = ', '.join(arg.implement for arg in self[Argument])
        s = (self._template and 'template<{}> '.format(self._template)) or ''
        s = s + str(self._typei)
        return s.format("{self.scope}{self._name}({argl})".format(self=self, argl=argl)) + ";"

    def ExistArgumentNamed(self, name):
        """Returns information about argument existence"""
        from Argument import Argument
        return self.hasChild(name=name, type=Argument)

    @property
    def template_types(self):
        """Returns the list of nested template types"""
        lt = self._template_types
        nt = super(Function, self).template_types
        lt.extend([x for x in nt if x not in self._template_types])
        return lt



# -*- coding: utf-8 -*-
import wx
from  ctx import theContext as context
import analytic.astd as astd
import model.file
from ToolAstExplorerPane import ToolAstExplorerPane
import app.resources as rc


class ast_explorer(wx.PyEvtHandler):
    """Class for providing ast explorer"""
    instance = None

    def __init__(self):
        """Initialize ast explorer"""
        super(ast_explorer, self).__init__()
        #create the menus
        self._menuid = context.frame.new_tool_id()
        self._imenu = wx.MenuItem(context.frame.menuTools, self._menuid, u"Ast explorer",
            u"show ast structure", wx.ITEM_NORMAL)
        context.frame.AddToolsMenu('Ast explorer', self._imenu)
        #bind the menu handlers
        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateAstExplorer, id=self._menuid)
        self.Bind(wx.EVT_MENU, self.OnAstExplorer, id=self._menuid)

    @classmethod
    def load(cls):
        """Setup tool for the environment"""
        return ast_explorer()

    def OnUpdateAstExplorer(self, event):
        """Handle the update"""
        # get the view book
        self.selected = None
        book = context.frame.viewBook
        try:
            view = book.GetCurrentPage()
            if view is not None:
                # get the selected item
                selected = view.selected
                if type(selected) is model.file.File:
                    if selected.project._language == 'python':
                        self.selected = selected
        except:
            pass
        event.Enable(bool(self.selected))

    def OnAstExplorer(self, event):
        """Handle the command"""
        item = self.selected
        path = item.abs_file
        try:
            with open(path, "r") as python_file:
                content = python_file.read()
                data = astd.parse(content)
        except:
            wx.MessageBox("The file contains errors", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, context.frame)
            return
        frame = context.frame
        p = ToolAstExplorerPane(frame.docBook, item, data)
        frame.docBook.Freeze()
        frame.docBook.AddPage(p, path, True, rc.GetBitmapIndex("mini_logo"))
        frame.docBook.Thaw()




# -*- coding: utf-8 -*-
import tran


class TransactionalMoveObject(tran.TransactionObject):
    """Implements a relationship move operation"""
    def __init__(self, **kwargs):
        """Initialice move"""
        self._object = kwargs['object']
        self._origin = kwargs['origin']
        self._target = kwargs['target']
        self._target_index = kwargs.get('index', 0)
        super(TransactionalMoveObject, self).__init__()
        #if the object has name, give the opportunity to add this name
        #to the current transaction name, so it will be visible in menu
        #take in amodelount that not format will remains after that, so
        #nested transactions will no see his object names included, if any
        if hasattr(self._object, '_name'):
            sname = tran.TransactionStack.instance.GetName()
            if sname:
                sname = sname.format(self._object._name)
                tran.TransactionStack.instance.SetName(sname)

    def OnUndoRedoAdd(self):
        """Process on undo redo add"""
        self._object.OnUndoRedoRemoving()  # this is a hack specific for move object
        self._object._parent = self._target
        if hasattr(self._object, '_parentIndex'):
            self._origin_index = self._object._parentIndex
        self._object._parentIndex = self._target_index
        self._object.OnUndoRedoAdd()
        self._target.OnUndoRedoChanged()
        super(TransactionalMoveObject, self).OnUndoRedoAdd()

    def OnUndoRedoRemoving(self):
        """Process on undo redo removed"""
        self._object.OnUndoRedoRemoving()
        self._object._parent = self._origin
        if hasattr(self, '_origin_index'):
            self._object._parentIndex = self._origin_index
        elif hasattr(self._object, '_parentIndex'):
            del self._object._parentIndex
        self._object.OnUndoRedoAdd()
        self._target.OnUndoRedoChanged()
        super(TransactionalMoveObject, self).OnUndoRedoAdd()





# -*- coding: utf-8 -*-

import tran


class ContextDeclaration(object):
    """Class decorator for context dclaration"""
    def __init__(self):
        """"""
        super(ContextDeclaration, self).__init__()

    def __call__(self, method):
        """"""
        def wrapped_call(*args, **kwargs):
            """Code for context method"""
            this = args[0]
            pf = args[1]
            if this._declare:
                path = this.path
                for element in path:
                    element.WriteContextsPrefixDeclaration(pf)
                method(*args, **kwargs)
                for element in path:
                    element.WriteContextsSufixDeclaration(pf)
        return wrapped_call


class ContextImplementation(object):
    """Class decorator for context implementation"""
    def __init__(self):
        """Initialice context for implementation"""
        super(ContextImplementation, self).__init__()

    def __call__(self, method):
        """"""
        def wrapped_call(*args, **kwargs):
            """Code for context method"""
            this = args[0]
            pf = args[1]
            if this._implement:
                path = this.path
                for element in path:
                    element.WriteContextsPrefixImplementation(pf)
                method(*args, **kwargs)
                for element in path:
                    element.WriteContextsSufixImplementation(pf)
        return wrapped_call


class ContextItem(tran.TransactionObject):
    """Implements a context item used"""
    def __init__(self, **kwargs):
        """Initialize context item"""
        self._name = kwargs.get('name', '')
        self._define = kwargs.get('define', '')
        self._enable = kwargs.get('enable', False)
        self._prefix_declaration = kwargs.get('prefix_declaration', '')
        self._sufix_declaration = kwargs.get('sufix_declaration', '')
        self._prefix_implementation = kwargs.get('prefix_implementation', '')
        self._sufix_implementation = kwargs.get('sufix_implementation', '')
        self._note = kwargs.get('note', '')
        super(ContextItem, self).__init__()

    def SaveState(self):
        """Do save state"""
        super(ContextItem, self).SaveState()

    def Delete(self):
        """Do a transactional delete for any object"""
        super(ContextItem, self).Delete()



# -*- coding: utf-8 -*-
"""
Created on Sun Dec 15 23:24:30 2013

@author: mel
"""
import model
import app.resources as rc


#def name(x):
#    """comparison"""
#    return x._name


class TypesFolder(model.Folder):
    """Implements a Folder representation"""
    class_container = False
    type_container = True
    function_container = False
    variable_container = False
    module_container = False
    namespace_container = False
    function_container = False
    member_container = False

    #visual methods
    def draggable(self):
        """returns info about if the object can be moved"""
        return False

    def __init__(self, **kwargs):
        """Initialization"""
        from Type import Type
        kwargs['name'] = 'Types'
        super(TypesFolder, self).__init__(**kwargs)
        #create basic read-only types
        for x in ['bool', 'char', 'char16_t', 'char32_t', 'double', 'float',
            'int', 'long', 'long long', 'long double', 'short',
            'unsigned char', 'unsigned int', 'unsigned long', 'unsigned long long',
            'unsigned short', 'void']:
            Type(parent=self, name=x, readonly=True)
        #add unnamed type for functions with implicit type
        Type(parent=self, name='', readonly=True, visibleInTree=False)
        #add @ type for on-the-fly template types
        Type(parent=self, name='@', readonly=True, visibleInTree=False)
        self[Type].sort(key=lambda x: x._name, reverse=False)
        self.update_container()

    def update_container(self):
        """Update the container info"""
        self.context_container = False
        self.folder_container = False
        self.diagram_container = False
        self.namespace_container = False

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folderT")

    def ExistMemberDataNamed(self, name):
        """Check recursively about the existence of nested child member"""
        return False





# -*- coding: utf-8 -*-

###########################################################################
## Python code generated with wxFormBuilder (version Jun  6 2014)
## http://www.wxformbuilder.org/
##
## PLEASE DO "NOT" EDIT THIS FILE!
###########################################################################

import wx
import wx.xrc
import wxx

###########################################################################
## Class TargetsView
###########################################################################

class TargetsView ( wx.Panel ):
	
	def __init__( self, parent ):
		wx.Panel.__init__ ( self, parent, id = wx.ID_ANY, pos = wx.DefaultPosition, size = wx.Size( 500,300 ), style = wx.TAB_TRAVERSAL|wx.WANTS_CHARS )
		
		self.SetExtraStyle( wx.WS_EX_BLOCK_EVENTS )
		
		fgSizer4 = wx.FlexGridSizer( 1, 1, 0, 0 )
		fgSizer4.AddGrowableCol( 0 )
		fgSizer4.AddGrowableRow( 0 )
		fgSizer4.SetFlexibleDirection( wx.BOTH )
		fgSizer4.SetNonFlexibleGrowMode( wx.FLEX_GROWMODE_SPECIFIED )
		
		self.m_tree = wxx.TreeCtrl( self, wx.ID_ANY, wx.DefaultPosition, wx.DefaultSize, wx.TR_HAS_BUTTONS|wx.TR_HIDE_ROOT|wx.TR_SINGLE )
		fgSizer4.Add( self.m_tree, 0, wx.ALL, 5 )
		
		
		self.SetSizer( fgSizer4 )
		self.Layout()
	
	def __del__( self ):
		pass
	




"""Subclass of ImportProject, which is generated by wxFormBuilder."""

import wx
import wxx
import app
import os
import model


# Implementing ImportProject
class ImportProjectDialog(app.ImportProject):
    """
    This dialog allows to import a project
    into the workspace.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, workspace):
        """Initialize dialog"""
        super(ImportProjectDialog, self).__init__(parent)
        self.workspace = workspace

    def CppImport(self, dlg=None):
        """Do the work of importing C++ files"""
        wx.MessageBox("The C++ import is not already implemented", "Error",
            wx.OK | wx.CENTER | wx.ICON_ERROR, self)
        return False

    def PythonImport(self, dlg=None):
        """Do the work of importing python files"""
        #create the project
        kwargs = {
            'parent': self.workspace, 'name': self.project_name, 'language': 'python',
            'dir': self.dir,
            'note': 'project imported from {self.origin_dir}'.format(self=self)}
        project = model.Project(**kwargs)
        return project.update_from_sources(dir=self.origin_dir, progress=dlg)

    def Validate(self):
        """Validate dialog"""
        # first of all, get the base dir
        self.origin_dir = self.m_dirPicker2.Path
        if not os.access(self.origin_dir, os.R_OK):
            wx.MessageBox("The path {self.origin_dir} is not readable".format(self=self), "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        (head, self.project_name) = os.path.split(self.origin_dir)
        if len(self.project_name) == 0:
            wx.MessageBox("The path {self.dir} is invalid".format(self=self), "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        self.dir = os.path.join(self.workspace._dir, self.project_name)
        #Operate throgh the import target
        self.option = self.m_choicebook4.GetSelection()
        return True

    def DoImport(self, dlg=None):
        """This call is expected to come after EndDialog(wx.ID_OK)"""
        if self.option == 0:
            return self.PythonImport(dlg)
        else:
            return self.CppImport(dlg)

    def OnCancel(self, event):
        """cancel event handler"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Handle OnOk"""
        if self.Validate():
            self.EndModal(wx.ID_OK)





# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
import re
import os
import model
import tran
import app.resources as rc


class Dir(model.TCommon, tran.TransactionFSObject):
    """Declares a file with transactional support"""
    def __init__(self, **kwargs):
        """init"""
        if 'name' not in kwargs:
            kwargs['name'] = os.path.split(kwargs['file'])[1]
        kwargs['uri'] = kwargs['parent'].project
        super(Dir, self).__init__(**kwargs)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folder")

    @property
    def bitmap_open_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folder_open")

    def Refresh(self):
        """Refresh the files contained in a sngle projec"""
        # The project itself represents the root directory
        # so we need to scan the root dir

        _dir = self.abs_file
        if not os.access(_dir, os.R_OK):
            return False
        # list all files excludding hidden files
        hidden = re.compile(r'\.(.)*')
        project = re.compile(r'(.)*\.pcc')
        excluded = [hidden, project]
        _fset = [s for s in os.listdir(_dir) if not any(y.match(s) for y in excluded)]
        knowns = self[model.file.File] + self[model.file.Dir]
        name_knowns = dict([(x._file, x) for x in knowns])
        kwargs = {'parent': self}
        for f in _fset:
            element = os.path.join(_dir, f)
            if element in name_knowns:
                name_knowns[element].Refresh()
                del name_knowns[element]
                continue
            kwargs['file'] = element
            if os.path.isdir(element):
                model.file.Dir(**kwargs).Refresh()
            else:
                model.file.File(**kwargs)
        # remove unexistent file
        for k in name_knowns:
            name_knowns[k].Delete()




"""Subclass of SelectLibraries, which is generated by wxFormBuilder."""

import wxx
from activity.models.ui import ui as ui
import model
from builtin import *


# Implementing SelectLibraries
class SelectLibrariesDialog(ui.SelectLibraries):
    """
    This dialog allows to add or remove
    libraries for the current project
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """initialize dialog"""
        super(SelectLibrariesDialog, self).__init__(parent)
        self._project = container.project
        self._libraries = self._project(model.LibrariesFolder, filter=lambda x: x.parent == self._project, cut=True)
        pos = 0
        for key in libraries:
            self.m_checkList6.Insert(key, pos, libraries[key])

    def get_kwargs(self):
        """Returns kwargs dictionary suitable for objects creation"""
        for item in range(0, self.m_checkList6.GetCount()):
            method = self.m_checkList6.GetClientData(item)
            method(self._libraries[0])
        return None




# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

"""
Created on Sun Dec 15 19:22:32 2013

@author: mel
"""
import model
import app.resources as rc


class PendingTasks(model.Folder):
    """Clase que representa a las tareas pendientes"""

    task_container = True

    @property
    def status_container(self):
        """return the status container"""
        return self

    def SetStatus(self, element):
        """Set the elements as pending"""
        if type(element) is model.tasks.Task:
            self.SaveState()
            self._status = 'pending'
            self._dateEnd = ''
        for subtask in element[model.tasks.Task]:
            self.SetStatus(subtask)
        for subtask in element[model.tasks.TaskFolder]:
            self.SetStatus(subtask)

    def __init__(self, **kwargs):
        """Inicializacion"""
        if 'name' not in kwargs:
            kwargs['name'] = 'Pending Tasks'
        kwargs['readonly'] = True
        super(PendingTasks, self).__init__(**kwargs)

    @property
    def can_delete(self):
        """Check abot if class can be deleted"""
        return super(PendingTasks, self).can_delete

    def Delete(self):
        """Delete diagram objects"""
        super(PendingTasks, self).Delete()

    def RemoveRelations(self):
        """Utility for undo/redo"""
        super(PendingTasks, self).RemoveRelations()

    def RestoreRelations(self):
        """Utility for undo/redo"""
        super(PendingTasks, self).RestoreRelations()

    def SaveState(self):
        """Utility for saving state"""
        super(PendingTasks, self).SaveState()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare object to delete"""
        super(PendingTasks, self).OnUndoRedoRemoving(root)

    def OnUndoRedoChanged(self):
        """Update from app"""
        #when the class is updated, ctor's and dtors, must be updated
        super(PendingTasks, self).OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        super(PendingTasks, self).OnUndoRedoAdd()

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('folder_pendings')

    @property
    def bitmap_open_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("folder_pendings_open")

    @property
    def label(self):
        """Get tree label"""
        return '{self._name}'.format(self=self)



"""Subclass of LogExplorerPane, which is generated by wxFormBuilder."""

import wx
import Tool
from ctx import context


# Implementing LogExplorerPane
class ToolLogExplorerPane(Tool.LogExplorerPane):
    """Panel de exploracion de operaciones"""
    def __init__(self, parent):
        """Inicializa dialogo"""
        Tool.LogExplorerPane.__init__(self, parent)
        self.app = context.app
        self.log = self.app.log
        sz = self.m_report.GetClientSize()
        w1 = sz.x // 4 - 4
        w2 = sz.x - w1 - 4
        self.m_report.InsertColumn(0, "date", wx.LIST_FORMAT_LEFT, w1)
        self.m_report.InsertColumn(1, "values", wx.LIST_FORMAT_LEFT, w2)
        for entry in self.log._log:
            self.m_report.Append((entry._time, entry._text))




# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:31:28 2013

@author: mel
"""

import model
import tran
import app.resources as rc


class Decorator(model.TComponent):
    """Implements Decorator representation"""
    argument_container = True

    def __init__(self, **kwargs):
        """Initialization"""
        self._call = kwargs.get("call", False)
        super(Decorator, self).__init__(**kwargs)
        container = self.outer_class or self.outer_module
        container._lastSrcTime = None
        container._lastHdrTime = None
        parent = self.parent
        if type(parent) is model.py.MemberMethod:
            if self.name == 'staticmethod':
                parent._staticmethod = self
            elif self._name == 'classmethod':
                parent._classmethod = self
            elif self._name == 'property':
                parent._property = self
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()

    def Delete(self):
        """Handle delete"""
        k = self.inner_module or self.inner_package
        super(Decorator, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        return super(Decorator, self).get_kwargs()

    def OnUndoRedoChanged(self):
        """Update from app"""
        self.parent.OnUndoRedoChanged()
        super(Decorator, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Do required actions for removing"""
        parent = self.parent
        if type(parent) is model.py.MemberMethod:
            if parent._staticmethod == self:
                parent._staticmethod = None
            elif parent._classmethod == self:
                parent._classmethod = None
            elif parent._property == self:
                parent._property = None
        super(Decorator, self).OnUndoRedoRemoving(root)
        self.parent.OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        self.parent.OnUndoRedoChanged()
        super(Decorator, self).OnUndoRedoAdd()
        parent = self.parent
        if type(parent) is model.py.MemberMethod:
            if self.name == 'staticmethod':
                parent._staticmethod = self
            elif self._name == 'classmethod':
                parent._classmethod = self
            elif self._name == 'property':
                parent._property = self

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('decorator')

    def ExportPythonCode(self, wf):
        """Export python code"""
        wf.writeln(self.label)

    @property
    def label(self):
        """Get tree label"""
        if self._call:
            from Argument import Argument
            from ArgsArgument import ArgsArgument
            from KwArgsArgument import KwArgsArgument
            alist = ', '.join(arg.label for arg in
                self[Argument] + self[ArgsArgument] + self[KwArgsArgument])
            return '@{self._name}({alist})'.format(self=self, alist=alist)
        else:
            return '@{self._name}'.format(self=self)







"""Subclass of Wait, which is generated by wxFormBuilder."""

import app


# Implementing Wait
class WaitDialog(app.Wait):
    """This class shows a dialog"""
    def __init__(self, parent):
        """Initialize dialog"""
        super(WaitDialog, self).__init__(parent)



# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-

"""Subclass of PyModulePane, which is generated by wxFormBuilder."""

#import wx
import wx, wx.stc, wxx, app
from ctx import theContext as context
import tran
import pane


# Implementing PyModulePane
class NavigatorPane(app.NavigatorPane):
    """Implements navigator pane"""
    def __init__(self, parent, mainframe):
        """Intialization of method editor"""
        super(NavigatorPane, self).__init__(parent)
        self._mainframe = mainframe
        self._url = None

    def PreDelete(self):
        """Remove toolbar first in order to avoid gtk-collision at close"""
        #self.m_auiToolBar5.Destroy()

    def NavigateTo(self, str_url):
        """Do navigation"""
        self._url = str_url
        self.m_url.Clear()
        self.m_url.WriteText(str_url)
        self.m_page.LoadURL(str_url)





# -*- coding: utf-8 -*-
"""This module defines a custom event for dispatching messages to logger window"""

import wx


class SearchEvent(wx.PyCommandEvent):
    """Custom event"""
    _type = wx.NewEventType()

    def __init__(self, line, fname, id=wx.ID_ANY):
        """Initializer"""
        self._line = line
        self._fname = fname
        wx.PyCommandEvent.__init__(self, SearchEvent._type, id)

    @property
    def line(self):
        """Returns the line"""
        return self._line

    @property
    def fname(self):
        """Returns the file"""
        return self._fname

EVT_SEARCH = wx.PyEventBinder(SearchEvent._type, 1)


# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-
"""

@author: mel
"""
import model


class Note(model.TComponent):
    """Implements class representation"""
    def __init__(self, **kwargs):
        """Initialization method"""
        kwargs['visibleInTree'] = False
        super(Note, self).__init__(**kwargs)

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs.update(super(Note, self).get_kwargs())
        return kwargs

    def Delete(self):
        """Delete diagram objects"""
        #process diagrams
        dias = self.project(model.ClassDiagram)
        for dia in dias:
            # Check if inherit is in
            elem = dia.FindElement(self)
            if elem is not None:
                dia.SaveState()
                dia.RemoveElement(elem)
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Note, self).Delete()

    def OnUndoRedoChanged(self):
        """Update from app"""
        #update class diagrams
        project = self.project
        if project is not None:
            dias = project(model.ClassDiagram)
            for dia in dias:
                # Check if class is in
                elem = dia.FindElement(self)
                if elem is not None:
                    elem.Layout()
                    if hasattr(dia, '_pane') and dia._pane is not None:
                        dia._pane.Refresh(False)
        super(Note, self).OnUndoRedoChanged()



from TextPane import TextPane
from FilePane import FilePane
from FilePythonPane import FilePythonPane
from Threads import Threads
from Locals import Locals
from Expressions import Expressions
from Breakpoints import Breakpoints
from StackFrame import StackFrame
from DebugCommand import DebugCommand



# -*- coding: utf-8 -*-

import wx

TEST_AGW = False

if TEST_AGW:
    import wx.lib.agw.aui as aui
    import wx.lib.agw.aui.auibook as base
else:
    import wx.aui as base


class AuiNotebook(base.AuiNotebook):
    """Aui notebook override due to interfaz/formbuilder collision"""

    def __init__(self, *args, **kwargs):
        """Initialize notebook"""
        super(AuiNotebook, self).__init__(*args, **kwargs)
        if TEST_AGW:
            self.SetAGWWindowStyleFlag(aui.AUI_NB_TAB_FLOAT | aui.AUI_NB_TAB_EXTERNAL_MOVE
            | aui.AUI_NB_TAB_MOVE | aui.AUI_NB_TAB_SPLIT | aui.AUI_NB_LEFT
            | aui.AUI_NB_CLOSE_ON_ACTIVE_TAB)

    def AddPage(self, *args, **kwargs):
        """We need to shortcut the mistake found to wx interface"""
        if TEST_AGW:
            img_index = None
            pane = args[0]
            if len(args) > 3:
                img_index = args[3]
                args = args[:3]
            if 'bitmap' in kwargs:
                img_index = kwargs['bitmap']
                del kwargs['bitmap']
            if 'disabled_bitmap' in kwargs:
                del kwargs['disabled_bitmap']
            if 'imageId' in kwargs:
                img_index = kwargs['bitmap']
            result = super(AuiNotebook, self).AddPage(*args, **kwargs)
            if img_index:
                page_index = self.GetPageIndex(pane)
                self.SetPageImage(page_index, img_index)
            return result
        if wx.__version__ >= '3.0.0.0':
            if len(args) >= 4:
                if args[3] is wx.NullBitmap:
                    l = list(args)
                    l[3] = wx.NOT_FOUND
                    args = tuple(l)
            else:
                l = list(args)
                l.append(wx.NOT_FOUND)
                args = tuple(l)
            bmp = kwargs.get('bitmap', None)
            if bmp:
                pane = args[0]
                del kwargs['bitmap']
            result = super(AuiNotebook, self).AddPage(*args, **kwargs)
            if bmp:
                page_index = self.GetPageIndex(pane)
                self.SetPageBitmap(page_index, bmp)
            return result


#    def GetCurrentPage(self, *args, **kwargs):
#        """Backward compatibility"""
#        if wx.__version < '3.0.0.0':




# -*- coding: utf-8 -*-
# -*- coding: utf-8 -*-
"""This module defines a custom event for dispatching messages to logger window"""

import wx

# Predefined event types
FILE_LINE_INFO = 1  # posted when the info line is returned
DEBUG_ENDED = 2     # posted when the debug session has end
UNKNOWN_DEBUG_INFO = 3  # posted when some remote message fails to get parsed
UPDATE_THREADS_INFO = 4  # posted when the threads info is available
UPDATE_LOCALS_INFO = 5  # posted when the locals info is available
UPDATE_BREAKPOINTS_INFO = 6  # posted when breakpoints info is available
USER_COMMAND_RESPONSE = 7  # posted while the reponse for an user command


class DebuggerEvent(wx.PyCommandEvent):
    """Custom event"""
    _type = wx.NewEventType()

    def __init__(self, wich, message=None, id=wx.ID_ANY):
        """Initializer"""
        self._wich = wich
        self._message = message
        wx.PyCommandEvent.__init__(self, DebuggerEvent._type, id)

    @property
    def wich(self):
        """Returns the message"""
        return self._wich

    @property
    def message(self):
        """Returns the message"""
        return self._message

EVT_DEBUGGER = wx.PyEventBinder(DebuggerEvent._type, 1)


"""Subclass of NewDir, which is generated by wxFormBuilder."""

import os
import wx
import wxx
import model
from activity.files.ui import ui as ui


# Implementing NewDir
class NewDir(ui.NewDir):
    """
    This dialog allows to create a new directory inside any project.
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent, container):
        """Initialize dialog"""
        """constructor"""
        self.container = container  # the receiver of the new element
        super(NewDir, self).__init__(parent)
        if type(container) is model.Project:
            self.m_dirPicker.SetPath(self.container.dir)
        else:
            self.m_dirPicker.SetPath(self.container._file)

    def Validate(self):
        """validate the dialog"""
        self._dir = self.m_dirPicker.GetPath()
        #check about dir existence
        if os.path.exists(self._dir):
            wx.MessageBox("Directory already exists", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    def CopyAttributes(self, fobj):
        """Copy dialog selections to object"""
        fobj._file = self._file

    def SetAttributes(self, fobj):
        """Copy object attributes to dialog"""
        self.m_dirPicker.SetPath(fobj.abs_file)

    def get_kwargs(self):
        """Returns suitable arguments for creating object"""
        kwargs = {'parent': self.container, 'file': self._dir}
        return kwargs

    def OnOK(self, event):
        """implement OnOK button"""
        if self.Validate():
            self.EndModal(wx.ID_OK)

    def OnCancel(self, event):
        """implement OnCancel button"""
        self.EndModal(wx.ID_CANCEL)



# -*- coding: utf-8 -*-
"""
Created on Mon Dec 16 08:28:25 2013

@author: mel
"""
import model
import model.cc
import tran
import app.resources as rc
from plugin import relationMethods, relationMembers, relationClasses
from tran import TransactionStack as ts


class Relation(tran.TransactionObject):
    """Implements a relation object"""
    # The Relation class is the hidden key of the both sides
    # This class is referenced by RelationFrom and RelationTo
    # by his _key attribute
    def __init__(self, **kwargs):
        """Init relation"""
        self._critical = kwargs.get('critical', False)
        self._global = kwargs.get('Global', False)
        self._filter = kwargs.get('filte', False)
        self._unique = kwargs.get('unique', False)
        self._unikey = kwargs.get('key', None)
        self._implementation = kwargs.get('implementation', "native")
        self._note = kwargs.get('note', "")
        self._project = kwargs['FROM'].project
        self._FROM = RelationFrom(
            name=kwargs['fromName'],
            parent=kwargs['TO'],
            FROM=kwargs['FROM'],
            access=kwargs.get('fromaccess', "public"),
            min=kwargs.get('frommin', None),
            max=kwargs.get('frommax', None),
            key=self)
        self._TO = RelationTo(
            name=kwargs['toName'],
            parent=kwargs['FROM'],
            TO=kwargs['TO'],
            access=kwargs.get('toaccess', "public"),
            min=kwargs.get('tomin', None),
            max=kwargs.get('tomax', None),
            key=self)
        # add relation to class diagrams
        for dia in self._project(model.ClassDiagram):
            child = dia.FindElement(kwargs['TO'])
            if child is None:
                continue
            parent = dia.FindElement(kwargs['FROM'])
            if parent is None:
                continue
            dia.SaveState()
            dia.AddRelation(self, parent, child)
        super(Relation, self).__init__()
        self.RecreateMembers()

    def RecreateMembers(self):
        """Do elements initialization"""
        self._FROM.RecreateMembers()
        self._TO.RecreateMembers()

    def init_dictionary(self, dictionary):
        """Inicializa el diccionario requerido por el parsing
        de las relaciones."""
        dictionary.update((x._name.replace(' ', '_'), x) for x in self._project.types)
        dictionary.update({
            'FROM': self._FROM._FROM,
            'TO': self._TO._TO,
            'fromRel': self._FROM,
            'toRel': self._TO
            })

    @property
    def project(self):
        """Gets the project"""
        return self._project

    @property
    def note(self):
        """Gets the comments"""
        return self._note

    @note.setter
    def note(self, value):
        """Sets the note"""
        self._note = value

    def SaveState(self, onlyRoot=False):
        """Save the current state in the undo stack"""
        if not onlyRoot:
            self._FROM.__SaveState__()
            self._TO.__SaveState__()
        super(Relation, self).SaveState()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        for dia in self._project(model.ClassDiagram):
            diarel = dia.FindElement(self)
            if diarel is not None:
                diarel.RefreshChanges()
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Relation, self).OnUndoRedoAdd()

    def OnUndoRedoChanged(self):
        """Handles changes"""
        if not ts.InUndoRedo():
            self.RecreateMembers()
        project = self.project
        if project is None:
            return
        dias = project(model.ClassDiagram)
        for dia in dias:
            diarel = dia.FindElement(self)
            if diarel is not None:
                diarel.RefreshChanges()
                if hasattr(dia, '_pane') and dia._pane is not None:
                    dia._pane.Refresh()
        super(Relation, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Prepare for delete"""
        super(Relation, self).OnUndoRedoRemoving()

    def Delete(self):
        """Transaction delete"""
        self._FROM.Delete()
        self._TO.Delete()
        project = self.project
        if project is not None:
            #process diagrams
            dias = project.diagrams
            for dia in dias:
                # Check if inherit is in
                elem = dia.FindElement(self)
                if elem is not None:
                    dia.SaveState()
                    dia.RemoveElement(elem)
                    if hasattr(dia, '_pane') and dia._pane is not None:
                        dia._pane.Refresh()
        self._FROM.parent.UpdateClassRelations()
        self._TO.parent.UpdateClassRelations()
        super(Relation, self).Delete()


class RelationFrom(model.TComponent):
    """Implements relation FROM"""
    def __init__(self, **kwargs):
        """Initialize the FROM relation"""
        self._FROM = kwargs['FROM']
        self._key = kwargs['key']
        self._access = kwargs.get('access', "public")
        self._minCardinal = kwargs.get('min', None)
        self._maxCardinal = kwargs.get('max', None)
        super(RelationFrom, self).__init__(**kwargs)

    def RecreateMembers(self):
        """Do extra project initialization"""
        #ensure no elements
        obj = []
        for cls in self._child:
            for element in self[cls]:
                obj.append(element)
        for element in obj:
            element.Delete()
        # recreate elements
        self._createRelationMembers()
        self._createRelationMethods()
        self._createRelationClasses()
        # update class initializers
        self.inner_class.UpdateClassRelations()

    def _createRelationMembers(self):
        """Create the relation members of the class"""
        #general dictionary
        dictionary = {}
        self._key.init_dictionary(dictionary)

        if self._key._implementation == "native":
            members = relationMembers(
                version='standard',
                filter={'transactional': False},
                part='FROM', dictionary=dictionary)

            if members is not None:
                for member in members:
                    #create type instances
                    member['type'] = model.cc.typeinst(**member)
                    member.update({'parent': self, 'readonly': True, 'update_ctors': False})
                    # The scripts need to access to some cross information
                    element = model.cc.MemberData(**member)
                    setattr(self, member['handler'], element)

    def _createRelationMethods(self):
        """Create the relation methods for the FROM side"""
        #general dictionary
        dictionary = {}
        self._key.init_dictionary(dictionary)
        if self._key._implementation == "native":
            methods = relationMethods(
                version='standard',
                filter={'transactional': False},
                part='FROM', dictionary=dictionary)

            #replace types
            if methods is not None:
                for method in methods:
                    #create type instances
                    key = method['key']
                    key['type'] = model.cc.typeinst(**key)
                    key['content'] = method['decl']
                    key.update({'parent': self, 'readonly': True})
                    m = model.cc.MemberMethod(**key)
                    for x in method['arglist']:
                        x['type'] = model.cc.typeinst(**x)
                        x.update({'parent': m, 'readonly': True})
                        model.cc.Argument(**x)

    def _createRelationClasses(self):
        """Create the relation classes for the FROM side"""
        #general dictionary
        dictionary = {}
        self._key.init_dictionary(dictionary)
        dictionary.update({'parent': self, 'readonly': True})
        if self._key._implementation == "native":
            relationClasses(
                version='standard',
                filter={'transactional': False},
                part='FROM', dictionary=dictionary)

    def GetFrom(self):
        """Amodeless FROM class"""
        return self._FROM

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("parent", self._access)

    def __SaveState__(self, onlyRoot=False):
        """Internal save state"""
        super(RelationFrom, self).SaveState()

    def SaveState(self, onlyRoot=False):
        """Redirects saving state to root"""
        self._key.SaveState(onlyRoot)


class RelationTo(model.TComponent):
    """Implements relation TO"""
    def __init__(self, **kwargs):
        """Initialize the TO relation"""
        self._TO = kwargs['TO']
        self._key = kwargs['key']
        self._access = kwargs.get('access', "public")
        self._minCardinal = kwargs.get('min', None)
        self._maxCardinal = kwargs.get('max', None)
        super(RelationTo, self).__init__(**kwargs)

    def RecreateMembers(self):
        """Do extra project initialization"""
        #ensure no elements
        obj = []
        for cls in self._child:
            for element in self[cls]:
                obj.append(element)
        for element in obj:
            element.Delete()
        # recreate elements
        self._createRelationMembers()
        self._createRelationMethods()
        self._createRelationClasses()
        self.inner_class.UpdateClassRelations()

    def _createRelationMembers(self):
        """Create the relation members of the class"""
        #general dictionary
        dictionary = {}
        self._key.init_dictionary(dictionary)

        if self._key._implementation == "native":
            members = relationMembers(
                version='standard',
                filter={'transactional': False},
                part='TO', dictionary=dictionary)

            if members is not None:
                for member in members:
                    #create type instances
                    member['type'] = model.cc.typeinst(**member)
                    member.update({'parent': self, 'readonly': True})
                    # The scripts need to access to some cross information
                    setattr(self, member['handler'], model.cc.MemberData(**member))

    def _createRelationMethods(self):
        """Create the relation methods for the FROM side
        """
        #general dictionary
        dictionary = {}
        self._key.init_dictionary(dictionary)

        if self._key._implementation == "native":
            methods = relationMethods(
                version='standard',
                filter={'transactional': False},
                part='TO', dictionary=dictionary)

            #replace types
            if methods is not None:
                for method in methods:
                    #create type instances
                    key = method['key']
                    key['type'] = model.cc.typeinst(**key)
                    key['content'] = method['decl']
                    key.update({'parent': self, 'readonly': True})
                    m = model.cc.MemberMethod(**key)
                    for x in method['arglist']:
                        x['type'] = model.cc.typeinst(**x)
                        x.update({'parent': m, 'readonly': True})
                        model.cc.Argument(**x)

    def _createRelationClasses(self):
        """Create the relation classes for the FROM side
        """
        #general dictionary
        dictionary = {}
        self._key.init_dictionary(dictionary)
        dictionary.update({'parent': self, 'readonly': True})
        if self._key._implementation == "native":
            relationClasses(
                version='standard',
                filter={'transactional': False},
                part='TO', dictionary=dictionary)

    def GetTo(self):
        """Amodeless TO class"""
        return self._TO

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex("child", self._access)

    def __SaveState__(self):
        """Internal save state"""
        super(RelationTo, self).SaveState()

    def SaveState(self, onlyRoot=False):
        """Redirects saving state to root"""
        self._key.SaveState(onlyRoot)



# -*- coding: utf-8 -*-

import os
import os.path
import sys
import signal
import re
import wx
import wxx
import threading
from ctx import localcontext as context
from activity.files.ui import pane


class threadInfo(object):
    """small class for thread information"""

    def __init__(self, **kwargs):
        """init"""
        self._is_current = kwargs.get('current', False)
        self._class = kwargs.get('cls', 'unknown')
        self._name = kwargs.get('name', '<unnamed>')
        self._status = kwargs.get('status', 'unknown')
        self._id = kwargs['id']
        self._stack = []
        super(threadInfo, self).__init__()

    @property
    def stack(self):
        """return thread stack"""
        return self._stack

    def append_stack(self, call, file, line):
        """Añade una llamada a la pila"""
        self._stack.append((call, file, line))

    @property
    def is_current(self):
        """return info about if the thread is the current thread"""
        return self._is_current

    @property
    def classname(self):
        """return the class name"""
        return self._class

    @property
    def name(self):
        """return the name"""
        return self._name

    @property
    def status(self):
        """return the thread status"""
        return self._status

    @property
    def id(self):
        """return the thread identifier"""
        return self._id


class debugPythonSession(object):
    """Handler of debug session"""

    def __init__(self, script=None):
        """Init"""
        self.ansi_escape = re.compile(r'\x1b[^m]*m')
        self.bin_obj = re.compile(r'<(?P<decl>.*?)>(?!\')')
        self.find_unquoted = [
            re.compile(r'\'copyright\': .*?\.\,', re.DOTALL),
            re.compile(r'\'credits\': .*?\.\,', re.DOTALL),
            re.compile(r'\'license\': .*?\,', re.DOTALL),
            re.compile(r'\'help\': .*?\.\,', re.DOTALL),
            re.compile(r'\'exit\': .*?\,', re.DOTALL),
            re.compile(r'\'quit\': .*?\,', re.DOTALL)
            ]        # hooks
        self.find_break = re.compile(
            r'(?P<bpnum>[0-9]+)\s+breakpoint\s+\w+\s+\w+\s+at\s+(?P<file>.+):(?P<line>[0-9]+)')
        self._set_file_line = lambda x, y: None
        # retained info
        self.info = {}
        self._process = None
        self._receiver = None
        self._message = []
        self._stepping = False
        self._threads = {}
        self._locals = {}
        self._breakpoints = {}
        self._buffer = ''  # buffer para stack
        self._bufferL = ''  # buffer para info de lineas
        self._bufferT = ''  # buffer para info de threads/locales
        self._bufferB = ''  # buffer para info de breakpoints
        self._sema = threading.Semaphore(0)
        super(debugPythonSession, self).__init__()
        if script:
            self.debugScript(script)

    @property
    def threads(self):
        """return the thread dictionary"""
        return self._threads

    @property
    def locals(self):
        """return the locals list"""
        return self._locals

    @property
    def breakpoints(self):
        """return the dict of breakpoints"""
        return self._breakpoints

    def activateUI(self):
        """Activate the panes"""
        context.frame.CleanOutputPane()
        self.addStackPane()
        self.addDebugCommandPane()
        self.activateOutputPane()
        self.activateWatchBook()

    def deactivateUI(self):
        """Deactivate the ui"""
        frame = context.frame
        self.removeStackPane()
        self.removeDebugCommandPane()
        frame.m_mgr.DetachPane(self.m_debugInfo)
        self.m_debugInfo.Destroy()
        frame.m_mgr.Update()

    def activateOutputPane(self):
        """Activate the output pane"""
        frame = context.frame
        if frame.m_aux_panes == frame.m_aux_output_pane:
            return True
        try:
            i = max(i for i in range(0, frame.m_aux_panes.GetPageCount())
            if frame.m_aux_panes.GetPage(i) == frame.m_aux_output_pane)
            frame.m_aux_panes.SetSelection(i)
            return True
        except:
            pass
        return False

    def removeStackPane(self):
        """Remove the stack pane"""
        frame = context.frame
        auxbook = frame.m_aux_panes
        for i in range(0, auxbook.GetPageCount()):
            if auxbook.GetPage(i) == self.m_stack:
                auxbook.DeletePage(i)
                return

    def removeDebugCommandPane(self):
        """Remove the debug command pane"""
        frame = context.frame
        auxbook = frame.m_aux_panes
        for i in range(0, auxbook.GetPageCount()):
            if auxbook.GetPage(i) == self.m_command:
                auxbook.DeletePage(i)
                return

    def addStackPane(self):
        """Add stackframe pane"""
        frame = context.frame
        self.m_stack = pane.StackFrame(frame.m_aux_panes, self)
        frame.m_aux_panes.AddPage(self.m_stack, u"stack frame", False)

    def addDebugCommandPane(self):
        """Add debug command pane"""
        frame = context.frame
        self.m_command = pane.DebugCommand(frame.m_aux_panes, self)
        frame.m_aux_panes.AddPage(self.m_command, u"debug command", False)

    def activateWatchBook(self):
        """Activate the watch panel"""
        frame = context.frame
        self.m_debugInfo = wx.aui.AuiNotebook(frame, wx.ID_ANY, wx.DefaultPosition,
            wx.Size(200, -1), wx.aui.AUI_NB_DEFAULT_STYLE)
        frame.m_mgr.AddPane(self.m_debugInfo, wx.aui.AuiPaneInfo().Right().PinButton(True).
            Dock().Resizable().FloatingSize(wx.Size(120, -1)).DockFixed(False).Row(1).Layer(4))
        # add panes:
        # threads and stack
        self.m_threads = pane.Threads(self.m_debugInfo, self)
        self.m_locals = pane.Locals(self.m_debugInfo, self)
        self.m_expressions = pane.Expressions(self.m_debugInfo, self)
        self.m_breakpoints = pane.Breakpoints(self.m_debugInfo, self)
        self.m_debugInfo.AddPage(self.m_threads, 'Threads', True, wx.NOT_FOUND)
        self.m_debugInfo.AddPage(self.m_locals, 'Locals', False, wx.NOT_FOUND)
        self.m_debugInfo.AddPage(self.m_expressions, 'Expressions', False, wx.NOT_FOUND)
        self.m_debugInfo.AddPage(self.m_breakpoints, 'Breakpoints', False, wx.NOT_FOUND)
        frame.m_mgr.Update()

    def UpdateThreadsUI(self):
        """Update the visual info"""
        self.m_threads.UpdateData()

    def UpdateDebugCommandUI(self, response):
        """Update debug command pane"""
        self.m_command.UpdateData(response)

    def UpdateLocalsUI(self):
        """Update the visual info"""
        self.m_locals.UpdateData()

    def UpdateBreakpointsUI(self):
        """Update the visual info"""
        self.m_breakpoints.UpdateData()

    def CaptureLocalsInfo(self, text):
        """capture lines of text"""
        self._buffer += str(text)

    def ProcessLocalsInfo(self):
        """Process the full thread info and dispatch"""
        text = self._buffer
        if not text:
            return
        self._buffer = ''
        cleaner = lambda x: '\'<{}>\''.format(re.sub(r'\'', '', x.group('decl')))
        text = self.bin_obj.sub(cleaner, text)
        for k in self.find_unquoted:
            text = k.sub('', text)
        #print '-----'
        #print text
        try:
            self._locals = eval(text)
        except:
            self._locals = {'note': 'failed translating locals'}

    def CaptureLineInfo(self, text):
        """obtiene info de las lineas"""
        self._bufferL += text

    def CaptureBreakInfo(self, text):
        """obtiene info de los breakpoints"""
        self._bufferB += text

    def ProcessBreakInfo(self):
        """procesa la informacion de los brekpoints"""
        text = self._bufferB
        if not text:
            return
        self._bufferB = ''
        lines = text.splitlines()
        self._breakpoints = {}
        for line in lines:
            z = self.find_break.match(line)
            if z:
                self._breakpoints[z.group('bpnum')] = (z.group('file'), z.group('line'))

    def CaptureThreadInfo(self, text):
        """capture linest of text"""
        self._bufferT += str(text)

    def ProcessLineInfo(self):
        """Process the line info"""
        text = self._bufferL
        if not text:
            return
        self._bufferL = ''
        text = ''.join(text.splitlines())
        handler = context.frame.GetEventHandler()
        #print 'processing line text {}'.format(text)
        z = re.search(r'Line (?P<lineno>[0-9]*) of \"(?P<file>.*)\"', text)
        if z:
            self._line = l = int(z.group('lineno'))
            self._file = f = z.group('file')
            # post a message
            handler.AddPendingEvent(wxx.DebuggerEvent(wxx.FILE_LINE_INFO, (f, l)))

    def ProcessThreadInfo(self):
        """Process the full thread info and dispatch"""
        text = self._bufferT
        if not text:
            return
        self._bufferT = ''
        handler = context.frame.GetEventHandler()
        lines = text.splitlines()[1:]  # discard separator
        thread_info_line = lines.pop(0)
        thread_info = re.search(r'(?P<ttype>..) <(?P<tclass>.+)\((?P<tname>\w+), (?P<tstatus>\w+) .*\)>: (?P<tid>[0-9]+)',
            thread_info_line)
        thread_current = False
        thread_class = '???'
        thread_name = '???'
        thread_status = '???'
        thread_id = '???'
        if thread_info:
            thread_current = (thread_info.group('ttype') == '=>')
            thread_class = thread_info.group('tclass')
            thread_name = thread_info.group('tname')
            thread_status = thread_info.group('tstatus')
            thread_id = thread_info.group('tid')
        else:
            thread_info = re.search(r'    thread id: (?P<tid>[0-9]+)', thread_info_line)
            if not thread_info:
                # post a info message
                msg = 'Thread info:{info}'.format(info=thread_info_line)
                handler.AddPendingEvent(wxx.DebuggerEvent(wxx.UNKNOWN_DEBUG_INFO, msg))
                #handler.AddPendingEvent(wxx.LoggerEvent(msg))
                return
        # The thread info is append to the internal list
        thread_id = thread_info.group('tid')
        thread = threadInfo(
            current=thread_current,
            cls=thread_class,
            name=thread_name,
            status=thread_status,
            id=thread_id)
        self._threads[thread_id] = thread
        # Now parse the stack frame
        header = True
        while len(lines):
            try:
                item = lines.pop(0).strip()
                if item[:8] == '<module>':
                    thread.append_stack(item, None, None)
                    continue
                _line = lines.pop(0).strip()
                # skip separator
                # dissmiss debugger lines
                if header:
                    if 'trepan.processor.command.info' in item:
                        continue
                    if 'trepan.processor.cmdproc' in item:
                        continue
                    if 'trepan.lib.core.DebuggerCore' in item:
                        continue
                    if item[:13] == '_tracer_func(':
                        continue
                header = False
                z = re.search(r"called from file '(?P<source>.+)' at line (?P<line>[0-9]+)", _line)
                if z:
                    s = '{f}:{l} in {c}'.format(
                        f=z.group('source'),
                        l=z.group('line'),
                        c=item)
                    thread.append_stack(s, z.group('source'), z.group('line'))
                else:
                    thread.append_stack(_line, None, None)
            except:
                break
        #print thread.stack

    def receiver_proc(self):
        """funcion que realiza el trabajo en el thread"""
        handler = context.frame.GetEventHandler()
        handler.AddPendingEvent(wxx.LoggerEvent('watching process ...'))
        line = ''
        for line in iter(self._process.stdout.readline, ""):
            handler.AddPendingEvent(wxx.LoggerEvent(line.strip()))
            self._process.stdout.flush()
            wx.YieldIfNeeded()
        handler.AddPendingEvent(wxx.LoggerEvent('closing ...'))
        # serious issue here: we cound'nt close the pipe!
        # closing that means closing the app stdout!
        #self._process.stdout.close()
        self._process.wait()
        self._process = None
        handler.AddPendingEvent(wxx.LoggerEvent('execution ended!'))
        handler.AddPendingEvent(wxx.DebuggerEvent(wxx.DEBUG_ENDED))

    def command_proc(self):
        """reads commands from the debugger"""
        from trepan.interfaces import comcodes as Mcomcodes
        from trepan.interfaces import client as Mclient
        connection_opts = {'open': True,
            'IO': 'TCP', 'HOST': '127.0.0.1', 'PORT': 8765}
        try:
            self.intf = Mclient.ClientInterface(connection_opts=connection_opts)
        except:
            attempts = 10
            while(attempts):
                try:
                    wx.Sleep(1)
                    self.intf = Mclient.ClientInterface(connection_opts=connection_opts)
                    break
                except:
                    attempts = attempts - 1
            if not attempts:
                frame = context.frame
                handler = frame.GetEventHandler()
                handler.AddPendingEvent(wxx.LoggerEvent('connection failed'))
                if self._process:
                    os.killpg(os.getpgid(self._process.pid), signal.SIGTERM)
                return

        self.done = False
        last_command = None
        handler = context.frame.GetEventHandler()
        with open(os.path.expanduser('~/.debug_log.log'), 'w') as dlog:
            dlog.write('starting session\n')
            while not self.done:
                try:
                    control, remote_msg = self.intf.read_remote()
                except:
                    # an exception has happen in the debugger
                    self.done = True
                    continue
                remote_msg = self.ansi_escape.sub('', remote_msg)  # remove ansi console color escape codes
                try:
                    if control == Mcomcodes.COMMAND:
                        cmd = 'COMMAND'
                    elif control == Mcomcodes.CONFIRM_FALSE:
                        cmd = 'CONFIRM_FALSE'
                    elif control == Mcomcodes.CONFIRM_REPLY:
                        cmd = 'CONFIRM_REPLY'
                    elif control == Mcomcodes.CONFIRM_TRUE:
                        cmd = 'CONFIRM_TRUE'
                    elif control == Mcomcodes.PRINT:
                        cmd = 'PRINT'
                    elif control == Mcomcodes.PROMPT:
                        cmd = 'PROMPT'
                    elif control == Mcomcodes.QUIT:
                        cmd = 'QUIT'
                    elif control == Mcomcodes.RESTART:
                        cmd = 'RESTART'
                    elif control == Mcomcodes.SYNC:
                        cmd = 'SYNC'
                    else:
                        cmd = 'UNKNOWN'
                    dlog.write('received remote msg: {d}[{m}] previous command: [{c}]\n'.format(
                        d=cmd, m=remote_msg, c=last_command or 'None'))
                except:
                    dlog.write('failed to write remote\n')
                # print 'c, r', control, remote_msg
                if control == Mcomcodes.PRINT:
                    if last_command:
                        if last_command[0] == '#':
                            handler.AddPendingEvent(wxx.DebuggerEvent(wxx.USER_COMMAND_RESPONSE, remote_msg))
                        elif last_command == 'info line':
                            dlog.write('capturing line\n')
                            self.CaptureLineInfo(remote_msg)
                        elif last_command == 'info threads verbose':
                            self.CaptureThreadInfo(remote_msg)
                        elif last_command == 'pp locals()':
                            self.CaptureLocalsInfo(remote_msg)
                        elif last_command == 'info break':
                            self.CaptureBreakInfo(remote_msg)
                        elif re.search(r'The program finished', remote_msg):
                            self.intf.write_remote(Mcomcodes.CONFIRM_REPLY, 'quit')
                    continue
                if control in [Mcomcodes.CONFIRM_TRUE, Mcomcodes.CONFIRM_FALSE]:
                    self.intf.write_remote(Mcomcodes.CONFIRM_REPLY, 'Y')
                    continue
                if control == Mcomcodes.PROMPT:
                    # here, we read the message
                    try:
                        if last_command == 'info line':
                            self.ProcessLineInfo()
                        elif last_command == 'info threads verbose':
                            self.ProcessThreadInfo()
                            handler.AddPendingEvent(wxx.DebuggerEvent(wxx.UPDATE_THREADS_INFO))
                        elif last_command == 'pp locals()':
                            self.ProcessLocalsInfo()
                            handler.AddPendingEvent(wxx.DebuggerEvent(wxx.UPDATE_LOCALS_INFO))
                        elif last_command == 'info break':
                            self.ProcessBreakInfo()
                            handler.AddPendingEvent(wxx.DebuggerEvent(wxx.UPDATE_BREAKPOINTS_INFO))
                    except:
                        pass
                    self._stepping = True
                    self._sema.acquire()
                    if not len(self._message):
                        continue
                    last_command = self._message[0]
                    del self._message[0]
                    self._stepping = False
                    # debug commands
                    #if last_command != 'info line':
                    #    handler.AddPendingEvent(wxx.LoggerEvent(last_command))
                    if last_command[0] == '#':
                        #this is an user command, using debug command pane
                        self.intf.write_remote(Mcomcodes.CONFIRM_REPLY, last_command[1:])
                    else:
                        self.intf.write_remote(Mcomcodes.CONFIRM_REPLY, last_command)
                    continue
                if control == Mcomcodes.QUIT:
                    self.done = True
                    continue
                if control == Mcomcodes.RESTART:
                    self.intf.inout.close()
                    wx.Sleep(1)
                    self.intf.inout.open()
                else:
                    print("!! Weird status code received '%s'" % control)
                    print(remote_msg,)
        self.intf.close()
        del self.intf

    def force_stop(self):
        """Brute stop"""
        self.done = True
        self._sema.release()
        if self._process:
            os.killpg(os.getpgid(self._process.pid), signal.SIGTERM)

    def send_command(self, msg):
        """Post a command for the debugger in the queue"""
        self._message.append(msg)
        self._sema.release()

    # el comando para el cliente es
    # python -m trepan.client --port=8765 {script}
    # ... esto es lo que hemos de hackear

    def debugScript(self, script):
        """Start a debug operation"""
        if self._process:
            return False  # is already debugging
        server_flags = [
            '--private', '--post-mortem', '--nx',
            '--port=8765', '--server']
        server_command = 'trepan2 {flags} {file}'.format(
            flags=' '.join(server_flags),
            file=script)
        import subprocess
        self.activateUI()
        try:
            self._process = subprocess.Popen(server_command,
                bufsize=1, stdout=subprocess.PIPE, stderr=subprocess.STDOUT, shell=True,
                preexec_fn=os.setsid)
            threading.Thread(target=self.receiver_proc,).start()
            threading.Thread(target=self.command_proc,).start()
        except:
            self.deeactivateUI()
            handler = context.frame.GetEventHandler()
            handler.AddPendingEvent(wxx.LoggerEvent('execution failed'))
            return False
        return True








# -*- coding: utf-8 -*-

# -*- coding: utf-8 -*-
"""
Created on Sun Dec 22 22:31:28 2013

@author: mel
"""

import model
import tran
import app.resources as rc


class Import(model.TComponent):
    """Implements argument representation"""

    context_container = False

    #visual methods
    @tran.TransactionalMethod('move import {}')
    def drop(self, to):
        """drop this elemento to another place"""
        target = to.inner_import_container
        if not target or to.project != self.project:
            return False  # avoid move arguments between projects
        index = 0  # trick for insert as first child
        tran.TransactionalMoveObject(
            object=self, origin=self.parent, target=target, index=index)
        return True

    def __init__(self, **kwargs):
        """Initialization"""
        self._as = kwargs.get('as', None)
        self._from = kwargs.get('from', None)  # (list of modules)
        super(Import, self).__init__(**kwargs)
        container = self.outer_class or self.outer_module
        container._lastSrcTime = None
        container._lastHdrTime = None
        k = self.inner_module or self.inner_package
        if k:
            k.ExportPythonCodeFiles()

    def Delete(self):
        """Handle delete"""
        k = self.inner_module or self.inner_package
        super(Import, self).Delete()
        if k:
            k.ExportPythonCodeFiles()

    def get_kwargs(self):
        """Returns the kwargs needed for this object"""
        kwargs = {}
        kwargs['as'] = self._as
        kwargs['from'] = self._from
        kwargs.update(super(Import, self).get_kwargs())
        return kwargs

    def OnUndoRedoChanged(self):
        """Update from app"""
        self.parent.OnUndoRedoChanged()
        super(Import, self).OnUndoRedoChanged()

    def OnUndoRedoRemoving(self, root=True):
        """Do required actions for removing"""
        super(Import, self).OnUndoRedoRemoving(root)
        self.parent.OnUndoRedoChanged()

    def OnUndoRedoAdd(self):
        """Restore object from undo"""
        self.parent.OnUndoRedoChanged()
        super(Import, self).OnUndoRedoAdd()

    def ExportPythonCode(self, wf):
        """Write code"""
        wf.writeln(self.label)

    @property
    def bitmap_index(self):
        """Index of tree image"""
        return rc.GetBitmapIndex('py_import')

    @property
    def label(self):
        """Get tree label"""
        if self._from:
            if self._as:
                return 'from {self._name} import {self._from} as {self._as}'.format(self=self)
            else:
                return 'from {self._name} import {self._from}'.format(self=self)
        if self._as:
            return 'import {self._name} as {self._as}'.format(self=self)
        return 'import {self._name}'.format(self=self)



"""Subclass of CommitGit, which is generated by wxFormBuilder."""

import wx
import wxx
from activity.git.ui import ui as ui
import app.resources as rc


# Implementing CommitGit
class CommitGitDialog(ui.CommitGit):
    """
    This dialog allows to commit the git repository
    """
    @wxx.SetInfo(__doc__)
    def __init__(self, parent):
        """Initialize the dialog"""
        super(CommitGitDialog, self).__init__(parent)

    def Validate(self):
        """Do dialog validation"""
        self._message = self.m_richText39.GetValue().strip()
        if len(self._message) == 0:
            wx.MessageBox("The commit message must be non empty", "Error",
                wx.OK | wx.CENTER | wx.ICON_ERROR, self)
            return False
        return True

    # Handlers for CommitGit events.
    def OnCancel(self, event):
        """Cancel the commit process"""
        self.EndModal(wx.ID_CANCEL)

    def OnOK(self, event):
        """Do the git process"""
        if self.Validate():
            self.EndModal(wx.ID_OK)
